//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2007 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       1.0.0
//  \   \        Filename:      autopilot.v
//  /   /        Date Created:  July 1, 2007
// /___/   /\    Last Modified: July 1, 2007
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Autopilot data storage
// Contact:   crabill@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (crabill) First created July 1, 2007.
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2007 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module autopilot
  (
  input  wire  [7:0] timestep,
  output wire [11:0] xlocation,
  output wire [11:0] ylocation,
  input  wire        clk
  );

  //******************************************************************//
  // Behavioral BlockRAM description.  At each address (timestep) the //
  // memory contains position information.                            //
  //******************************************************************//

  reg  [23:0] rompilot;

  assign xlocation = rompilot[11: 0];
  assign ylocation = rompilot[23:12];

  always @(posedge clk)
  begin
    case (timestep)
      0  : rompilot <= {12'd1200    , 12'd0       };
      1  : rompilot <= {12'd1100    , 12'd0       };
      2  : rompilot <= {12'd1082    , 12'd49      };
      3  : rompilot <= {12'd1057    , 12'd98      };
      4  : rompilot <= {12'd1029    , 12'd145     };
      5  : rompilot <= {12'd1000    , 12'd191     };
      6  : rompilot <= {12'd971     , 12'd237     };
      7  : rompilot <= {12'd942     , 12'd281     };
      8  : rompilot <= {12'd914     , 12'd324     };
      9  : rompilot <= {12'd886     , 12'd366     };
      10 : rompilot <= {12'd860     , 12'd407     };
      11 : rompilot <= {12'd834     , 12'd447     };
      12 : rompilot <= {12'd809     , 12'd486     };
      13 : rompilot <= {12'd785     , 12'd523     };
      14 : rompilot <= {12'd762     , 12'd560     };
      15 : rompilot <= {12'd739     , 12'd596     };
      16 : rompilot <= {12'd718     , 12'd630     };
      17 : rompilot <= {12'd697     , 12'd664     };
      18 : rompilot <= {12'd678     , 12'd696     };
      19 : rompilot <= {12'd659     , 12'd727     };
      20 : rompilot <= {12'd640     , 12'd758     };
      21 : rompilot <= {12'd623     , 12'd787     };
      22 : rompilot <= {12'd606     , 12'd815     };
      23 : rompilot <= {12'd589     , 12'd842     };
      24 : rompilot <= {12'd574     , 12'd868     };
      25 : rompilot <= {12'd559     , 12'd893     };
      26 : rompilot <= {12'd544     , 12'd917     };
      27 : rompilot <= {12'd530     , 12'd940     };
      28 : rompilot <= {12'd517     , 12'd961     };
      29 : rompilot <= {12'd504     , 12'd982     };
      30 : rompilot <= {12'd492     , 12'd1002    };
      31 : rompilot <= {12'd480     , 12'd1020    };
      32 : rompilot <= {12'd468     , 12'd1038    };
      33 : rompilot <= {12'd457     , 12'd1054    };
      34 : rompilot <= {12'd446     , 12'd1070    };
      35 : rompilot <= {12'd436     , 12'd1084    };
      36 : rompilot <= {12'd426     , 12'd1097    };
      37 : rompilot <= {12'd417     , 12'd1109    };
      38 : rompilot <= {12'd407     , 12'd1120    };
      39 : rompilot <= {12'd399     , 12'd1130    };
      40 : rompilot <= {12'd390     , 12'd1139    };
      41 : rompilot <= {12'd382     , 12'd1147    };
      42 : rompilot <= {12'd374     , 12'd1154    };
      43 : rompilot <= {12'd366     , 12'd1160    };
      44 : rompilot <= {12'd359     , 12'd1164    };
      45 : rompilot <= {12'd352     , 12'd1168    };
      46 : rompilot <= {12'd345     , 12'd1171    };
      47 : rompilot <= {12'd338     , 12'd1172    };
      48 : rompilot <= {12'd332     , 12'd1173    };
      49 : rompilot <= {12'd325     , 12'd1172    };
      50 : rompilot <= {12'd318     , 12'd1170    };
      51 : rompilot <= {12'd311     , 12'd1168    };
      52 : rompilot <= {12'd305     , 12'd1164    };
      53 : rompilot <= {12'd298     , 12'd1159    };
      54 : rompilot <= {12'd292     , 12'd1153    };
      55 : rompilot <= {12'd287     , 12'd1146    };
      56 : rompilot <= {12'd281     , 12'd1138    };
      57 : rompilot <= {12'd276     , 12'd1129    };
      58 : rompilot <= {12'd271     , 12'd1118    };
      59 : rompilot <= {12'd266     , 12'd1107    };
      60 : rompilot <= {12'd261     , 12'd1095    };
      61 : rompilot <= {12'd256     , 12'd1081    };
      62 : rompilot <= {12'd252     , 12'd1067    };
      63 : rompilot <= {12'd247     , 12'd1051    };
      64 : rompilot <= {12'd243     , 12'd1035    };
      65 : rompilot <= {12'd239     , 12'd1017    };
      66 : rompilot <= {12'd235     , 12'd998     };
      67 : rompilot <= {12'd231     , 12'd978     };
      68 : rompilot <= {12'd227     , 12'd957     };
      69 : rompilot <= {12'd224     , 12'd935     };
      70 : rompilot <= {12'd220     , 12'd912     };
      71 : rompilot <= {12'd217     , 12'd888     };
      72 : rompilot <= {12'd214     , 12'd863     };
      73 : rompilot <= {12'd210     , 12'd837     };
      74 : rompilot <= {12'd207     , 12'd810     };
      75 : rompilot <= {12'd207     , 12'd789     };
      76 : rompilot <= {12'd210     , 12'd762     };
      77 : rompilot <= {12'd214     , 12'd736     };
      78 : rompilot <= {12'd217     , 12'd711     };
      79 : rompilot <= {12'd220     , 12'd687     };
      80 : rompilot <= {12'd224     , 12'd664     };
      81 : rompilot <= {12'd227     , 12'd642     };
      82 : rompilot <= {12'd231     , 12'd621     };
      83 : rompilot <= {12'd235     , 12'd601     };
      84 : rompilot <= {12'd239     , 12'd582     };
      85 : rompilot <= {12'd243     , 12'd564     };
      86 : rompilot <= {12'd247     , 12'd548     };
      87 : rompilot <= {12'd252     , 12'd532     };
      88 : rompilot <= {12'd256     , 12'd518     };
      89 : rompilot <= {12'd261     , 12'd504     };
      90 : rompilot <= {12'd266     , 12'd492     };
      91 : rompilot <= {12'd271     , 12'd481     };
      92 : rompilot <= {12'd276     , 12'd470     };
      93 : rompilot <= {12'd281     , 12'd461     };
      94 : rompilot <= {12'd287     , 12'd453     };
      95 : rompilot <= {12'd292     , 12'd446     };
      96 : rompilot <= {12'd298     , 12'd440     };
      97 : rompilot <= {12'd305     , 12'd435     };
      98 : rompilot <= {12'd311     , 12'd432     };
      99 : rompilot <= {12'd318     , 12'd429     };
      100: rompilot <= {12'd325     , 12'd427     };
      101: rompilot <= {12'd332     , 12'd426     };
      102: rompilot <= {12'd338     , 12'd427     };
      103: rompilot <= {12'd345     , 12'd428     };
      104: rompilot <= {12'd352     , 12'd431     };
      105: rompilot <= {12'd359     , 12'd435     };
      106: rompilot <= {12'd366     , 12'd439     };
      107: rompilot <= {12'd374     , 12'd445     };
      108: rompilot <= {12'd382     , 12'd452     };
      109: rompilot <= {12'd390     , 12'd460     };
      110: rompilot <= {12'd399     , 12'd469     };
      111: rompilot <= {12'd407     , 12'd479     };
      112: rompilot <= {12'd417     , 12'd490     };
      113: rompilot <= {12'd426     , 12'd502     };
      114: rompilot <= {12'd436     , 12'd515     };
      115: rompilot <= {12'd446     , 12'd529     };
      116: rompilot <= {12'd457     , 12'd545     };
      117: rompilot <= {12'd468     , 12'd561     };
      118: rompilot <= {12'd480     , 12'd579     };
      119: rompilot <= {12'd492     , 12'd597     };
      120: rompilot <= {12'd504     , 12'd617     };
      121: rompilot <= {12'd517     , 12'd638     };
      122: rompilot <= {12'd530     , 12'd659     };
      123: rompilot <= {12'd544     , 12'd682     };
      124: rompilot <= {12'd559     , 12'd706     };
      125: rompilot <= {12'd574     , 12'd731     };
      126: rompilot <= {12'd589     , 12'd757     };
      127: rompilot <= {12'd606     , 12'd784     };
      128: rompilot <= {12'd623     , 12'd812     };
      129: rompilot <= {12'd640     , 12'd841     };
      130: rompilot <= {12'd659     , 12'd872     };
      131: rompilot <= {12'd678     , 12'd903     };
      132: rompilot <= {12'd697     , 12'd935     };
      133: rompilot <= {12'd718     , 12'd969     };
      134: rompilot <= {12'd739     , 12'd1003    };
      135: rompilot <= {12'd762     , 12'd1039    };
      136: rompilot <= {12'd785     , 12'd1076    };
      137: rompilot <= {12'd809     , 12'd1113    };
      138: rompilot <= {12'd834     , 12'd1152    };
      139: rompilot <= {12'd860     , 12'd1192    };
      140: rompilot <= {12'd886     , 12'd1233    };
      141: rompilot <= {12'd914     , 12'd1275    };
      142: rompilot <= {12'd942     , 12'd1318    };
      143: rompilot <= {12'd971     , 12'd1362    };
      144: rompilot <= {12'd1000    , 12'd1408    };
      145: rompilot <= {12'd1029    , 12'd1454    };
      146: rompilot <= {12'd1057    , 12'd1501    };
      147: rompilot <= {12'd1082    , 12'd1550    };
      148: rompilot <= {12'd1100    , 12'd1599    };
      149: rompilot <= {12'd1200    , 12'd1600    };
      // This covers all the unlisted cases.
      default: rompilot <= {12'd1200    , 12'd1600    };
    endcase
  end
 
  //******************************************************************//
  //                                                                  //
  //******************************************************************//

endmodule