//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2007 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor:        Xilinx
// \   \   \/    Version:       1.0.0
//  \   \        Filename:      cursor_pair.v
//  /   /        Date Created:  July 1, 2007
// /___/   /\    Last Modified: July 1, 2007
// \   \  /  \
//  \___\/\___\
//
// Devices:   Spartan-3 Generation FPGA
// Purpose:   Dual-ported block ROM
// Contact:   crabill@xilinx.com
// Reference: None
//
// Revision History:
//   Rev 1.0.0 - (crabill) First created July 1, 2007.
//
//////////////////////////////////////////////////////////////////////////////
//
// LIMITED WARRANTY AND DISCLAIMER. These designs are provided to you "as is".
// Xilinx and its licensors make and you receive no warranties or conditions,
// express, implied, statutory or otherwise, and Xilinx specifically disclaims
// any implied warranties of merchantability, non-infringement, or fitness for
// a particular purpose. Xilinx does not warrant that the functions contained
// in these designs will meet your requirements, or that the operation of
// these designs will be uninterrupted or error free, or that defects in the
// designs will be corrected. Furthermore, Xilinx does not warrant or make any
// representations regarding use or the results of the use of the designs in
// terms of correctness, accuracy, reliability, or otherwise.
//
// LIMITATION OF LIABILITY. In no event will Xilinx or its licensors be liable
// for any loss of data, lost profits, cost or procurement of substitute goods
// or services, or for any special, incidental, consequential, or indirect
// damages arising from the use or operation of the designs or accompanying
// documentation, however caused and on any theory of liability. This
// limitation will apply even if Xilinx has been advised of the possibility
// of such damage. This limitation shall apply not-withstanding the failure
// of the essential purpose of any limited remedies herein.
//
//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2007 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

`timescale 1 ns / 1 ps

module cursor_pair
  #(
  parameter          IMAGE = "XILINX"
  )
  (
  input  wire [11:0] xidx0,
  input  wire [11:0] yidx0,
  output reg   [1:0] data0,
  output reg         mask0,
  input  wire [11:0] xidx1,
  input  wire [11:0] yidx1,
  output reg   [1:0] data1,
  output reg         mask1,
  input  wire        clk
  );

  //******************************************************************//
  // Generate address signals and other helpers...                    //
  //******************************************************************//

  wire   [13:0] idx0_addr;
  wire          idx0_maskx;
  wire          idx0_masky;

  assign idx0_addr = {yidx0[ 6:0],xidx0[ 6:0]};
  assign idx0_maskx = | xidx0[11:7];
  assign idx0_masky = | yidx0[11:7];

  wire   [13:0] idx1_addr;
  wire          idx1_maskx;
  wire          idx1_masky;

  assign idx1_addr = {yidx1[ 6:0],xidx1[ 6:0]};
  assign idx1_maskx = | xidx1[11:7];
  assign idx1_masky = | yidx1[11:7];

  //******************************************************************//
  // Both ports are organized as x2 data output.                      //
  //******************************************************************//
 
  wire [1:0] hwc_al_out;
  wire [1:0] hwc_bl_out;
  wire [1:0] hwc_ah_out;
  wire [1:0] hwc_bh_out;

  generate
    if (IMAGE == "XILINX")
    begin : xilinx_logo
 
      RAMB16_S2_S2 #(
        .INIT_00 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_01 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_02 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_03 (256'h5AAAAAAAAAAAAAAAAAAAAA5555555556AAAAAAAAAAAAAAAAAAAA555555555555),
        .INIT_04 (256'h56AAAAAAAAAAAAAAAAAAAA955555555AAAAAAAAAAAAAAAAAAAAA955555555555),
        .INIT_05 (256'h55AAAAAAAAAAAAAAAAAAAAA55555556A96AAAAAAAAAAAAAAAAAAA55555555555),
        .INIT_06 (256'h556AAAAAAAAAAAAAAAAAAAA9555555AA55AAAAAAAAAAAAAAAAAAA95555555555),
        .INIT_07 (256'h555AAAAAAAAAAAAAAAAAAAAA555556A9556AAAAAAAAAAAAAAAAAAA5555555555),
        .INIT_08 (256'h5556AAAAAAAAAAAAAAAAAAAA95555AA5555AAAAAAAAAAAAAAAAAAA9555555555),
        .INIT_09 (256'h5555AAAAAAAAAAAAAAAAAAAAA5556A955556AAAAAAAAAAAAAAAAAAA555555555),
        .INIT_0A (256'h55556AAAAAAAAAAAAAAAAAAAA955AA555555AAAAAAAAAAAAAAAAAAA955555555),
        .INIT_0B (256'h55555AAAAAAAAAAAAAAAAAAAAA56A95555556AAAAAAAAAAAAAAAAAAA55555555),
        .INIT_0C (256'h555556AAAAAAAAAAAAAAAAAAAA9AA55555555AAAAAAAAAAAAAAAAAAA95555555),
        .INIT_0D (256'h555555AAAAAAAAAAAAAAAAAAAAAA9555555556AAAAAAAAAAAAAAAAAAA5555555),
        .INIT_0E (256'h5555556AAAAAAAAAAAAAAAAAAAAA5555555555AAAAAAAAAAAAAAAAAAA9555555),
        .INIT_0F (256'h5555555AAAAAAAAAAAAAAAAAAAA955555555556AAAAAAAAAAAAAAAAAAA555555),
        .INIT_10 (256'h55555556AAAAAAAAAAAAAAAAAAA555555555555AAAAAAAAAAAAAAAAAAA955555),
        .INIT_11 (256'h55555555AAAAAAAAAAAAAAAAAA95555555555556AAAAAAAAAAAAAAAAAAA55555),
        .INIT_12 (256'h555555556AAAAAAAAAAAAAAAAA55555555555555AAAAAAAAAAAAAAAAAAA95555),
        .INIT_13 (256'h555555555AAAAAAAAAAAAAAAA9555555555555556AAAAAAAAAAAAAAAAAAA5555),
        .INIT_14 (256'h5555555556AAAAAAAAAAAAAAA5555555555555555AAAAAAAAAAAAAAAAAAA9555),
        .INIT_15 (256'h5555555555AAAAAAAAAAAAAA955555555555555556AAAAAAAAAAAAAAAAAAA555),
        .INIT_16 (256'h55555555556AAAAAAAAAAAAA555555555555555555AAAAAAAAAAAAAAAAAAA955),
        .INIT_17 (256'h55555555555AAAAAAAAAAAA95555555555555555556AAAAAAAAAAAAAAAAAAA55),
        .INIT_18 (256'h555555555556AAAAAAAAAAA55555555555555555555AAAAAAAAAAAAAAAAAAA95),
        .INIT_19 (256'h555555555555AAAAAAAAAA9555555555555555555556AAAAAAAAAAAAAAAAAAA5),
        .INIT_1A (256'h5555555555556AAAAAAAAA5555555555555555555555AAAAAAAAAAAAAAAAAAA9),
        .INIT_1B (256'h5555555555555AAAAAAAA955555555555555555555556AAAAAAAAAAAAAAAAAAA),
        .INIT_1C (256'h55555555555556AAAAAAA55555555555555555555555AAAAAAAAAAAAAAAAAAA9),
        .INIT_1D (256'h55555555555555AAAAAA955555555555555555555556AAAAAAAAAAAAAAAAAAA5),
        .INIT_1E (256'h555555555555556AAAAA55555555555555555555555AAAAAAAAAAAAAAAAAAA95),
        .INIT_1F (256'h555555555555555AAAA955555555555555555555556AAAAAAAAAAAAAAAAAAA55),
        .INIT_20 (256'h5555555555555556AAA55555555555555555555555AAAAAAAAAAAAAAAAAAA955),
        .INIT_21 (256'h5555555555555555AA955555555555555555555556AAAAAAAAAAAAAAAAAAA555),
        .INIT_22 (256'h55555555555555556A55555555555555555555555AAAAAAAAAAAAAAAAAAA9555),
        .INIT_23 (256'h55555555555555555955555555555555555555556AAAAAAAAAAAAAAAAAAA5555),
        .INIT_24 (256'h5555555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA95555),
        .INIT_25 (256'h5555555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA55555),
        .INIT_26 (256'h555555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA955555),
        .INIT_27 (256'h555555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA555555),
        .INIT_28 (256'h55555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA9555555),
        .INIT_29 (256'h55555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA5555555),
        .INIT_2A (256'h5555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA95555555),
        .INIT_2B (256'h5555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA55555555),
        .INIT_2C (256'h555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA955555555),
        .INIT_2D (256'h555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA555555555),
        .INIT_2E (256'h55555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA9555555555),
        .INIT_2F (256'h55555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA5555555555),
        .INIT_30 (256'h5555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA95555555555),
        .INIT_31 (256'h5555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA55555555555),
        .INIT_32 (256'h555555555555555555555555555555555AAAAAAAAAAAAAAAAAAA955555555555),
        .INIT_33 (256'h555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA555555555555),
        .INIT_34 (256'h55555555555555555555555555555555AAAAAAAAAAAAAAAAAAA9555555555555),
        .INIT_35 (256'h55555555555555555555555555555556AAAAAAAAAAAAAAAAAAA5555555555555),
        .INIT_36 (256'h5555555555555555555555555555555AAAAAAAAAAAAAAAAAAA95555555555555),
        .INIT_37 (256'h5555555555555555555555555555556AAAAAAAAAAAAAAAAAAA55555555555555),
        .INIT_38 (256'h555555555555555555555555555555AAAAAAAAAAAAAAAAAAA955555555555555),
        .INIT_39 (256'h555555555555555555555555555556AAAAAAAAAAAAAAAAAAA555555555555555),
        .INIT_3A (256'h55555555555555555555555555555AAAAAAAAAAAAAAAAAAA9555555555555555),
        .INIT_3B (256'h55555555555555555555555555556AAAAAAAAAAAAAAAAAAA5555555555555555),
        .INIT_3C (256'h5555555555555555555555555555AAAAAAAAAAAAAAAAAAA95555555555555555),
        .INIT_3D (256'h5555555555555555555555555556AAAAAAAAAAAAAAAAAAA55555555555555555),
        .INIT_3E (256'h555555555555555555555555555AAAAAAAAAAAAAAAAAAA955555555555555555),
        .INIT_3F (256'h555555555555555555555555556AAAAAAAAAAAAAAAAAAA955555555555555555))
      HWCA (
        .DOA(hwc_al_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bl_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

      RAMB16_S2_S2 #(
        .INIT_00 (256'h555555555555555555555555555AAAAAAAAAAAAAAAAAAAA55555555555555555),
        .INIT_01 (256'h5555555555555555555555555556AAAAAAAAAAAAAAAAAAA95555555555555555),
        .INIT_02 (256'h5555555555555555555555555555AAAAAAAAAAAAAAAAAAAA5555555555555555),
        .INIT_03 (256'h55555555555555555555555555556AAAAAAAAAAAAAAAAAAA9555555555555555),
        .INIT_04 (256'h55555555555555555555555555555AAAAAAAAAAAAAAAAAAAA555555555555555),
        .INIT_05 (256'h555555555555555555555555555556AAAAAAAAAAAAAAAAAAA955555555555555),
        .INIT_06 (256'h555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA55555555555555),
        .INIT_07 (256'h5555555555555555555555555555556AAAAAAAAAAAAAAAAAAA95555555555555),
        .INIT_08 (256'h5555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA5555555555555),
        .INIT_09 (256'h55555555555555555555555555555556AAAAAAAAAAAAAAAAAAA9555555555555),
        .INIT_0A (256'h55555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA555555555555),
        .INIT_0B (256'h555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA955555555555),
        .INIT_0C (256'h555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA55555555555),
        .INIT_0D (256'h5555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA95555555555),
        .INIT_0E (256'h5555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA5555555555),
        .INIT_0F (256'h55555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA9555555555),
        .INIT_10 (256'h55555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA555555555),
        .INIT_11 (256'h555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA955555555),
        .INIT_12 (256'h555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA55555555),
        .INIT_13 (256'h5555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA95555555),
        .INIT_14 (256'h5555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA5555555),
        .INIT_15 (256'h55555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA9555555),
        .INIT_16 (256'h55555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA555555),
        .INIT_17 (256'h555555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA955555),
        .INIT_18 (256'h555555555555555555555555555555555555555AAAAAAAAAAAAAAAAAAAA55555),
        .INIT_19 (256'h5555555555555555555555555555555555555556AAAAAAAAAAAAAAAAAAA95555),
        .INIT_1A (256'h5555555555555555655555555555555555555555AAAAAAAAAAAAAAAAAAAA5555),
        .INIT_1B (256'h5555555555555555A955555555555555555555556AAAAAAAAAAAAAAAAAAA9555),
        .INIT_1C (256'h5555555555555556AA55555555555555555555555AAAAAAAAAAAAAAAAAAAA555),
        .INIT_1D (256'h555555555555555AAA955555555555555555555556AAAAAAAAAAAAAAAAAAA955),
        .INIT_1E (256'h555555555555556AAAA55555555555555555555555AAAAAAAAAAAAAAAAAAAA55),
        .INIT_1F (256'h55555555555555AAAAA955555555555555555555556AAAAAAAAAAAAAAAAAAA95),
        .INIT_20 (256'h55555555555556AAAAAA55555555555555555555555AAAAAAAAAAAAAAAAAAAA5),
        .INIT_21 (256'h5555555555555AAAAAAA955555555555555555555556AAAAAAAAAAAAAAAAAAA9),
        .INIT_22 (256'h5555555555556AAAAAAAA55555555555555555555555AAAAAAAAAAAAAAAAAAAA),
        .INIT_23 (256'h555555555555AAAAAAAAA95555555555555555555555AAAAAAAAAAAAAAAAAAA9),
        .INIT_24 (256'h555555555556AAAAAAAAAA5555555555555555555556AAAAAAAAAAAAAAAAAAA5),
        .INIT_25 (256'h55555555555AAAAAAAAAAA955555555555555555555AAAAAAAAAAAAAAAAAAA95),
        .INIT_26 (256'h55555555556AAAAAAAAAAAA55555555555555555556AAAAAAAAAAAAAAAAAAA55),
        .INIT_27 (256'h5555555555AAAAAAAAAAAAA9555555555555555555AAAAAAAAAAAAAAAAAAA955),
        .INIT_28 (256'h5555555556AAAAAAAAAAAAAA555555555555555556AAAAAAAAAAAAAAAAAAA555),
        .INIT_29 (256'h555555555AAAAAAAAAAAAAAA95555555555555555AAAAAAAAAAAAAAAAAAA9555),
        .INIT_2A (256'h555555556AAAAAAAAAAAAAAAA5555555555555556AAAAAAAAAAAAAAAAAAA5555),
        .INIT_2B (256'h55555555AAAAAAAAAAAAAAAAA955555555555555AAAAAAAAAAAAAAAAAAA95555),
        .INIT_2C (256'h55555556AAAAAAAAAAAAAAAAAA55555555555556AAAAAAAAAAAAAAAAAAA55555),
        .INIT_2D (256'h5555555AAAAAAAAAAAAAAAAAAA9555555555555AAAAAAAAAAAAAAAAAAA955555),
        .INIT_2E (256'h5555556AAAAAAAAAAAAAAAAAAAA555555555556AAAAAAAAAAAAAAAAAAA555555),
        .INIT_2F (256'h555555AAAAAAAAAAAAAAAAAAAAA95555555555AAAAAAAAAAAAAAAAAAA9555555),
        .INIT_30 (256'h555556AAAAAAAAAAAAAAAAAAAAAA5555555556AAAAAAAAAAAAAAAAAAA5555555),
        .INIT_31 (256'h55555AAAAAAAAAAAAAAAAAAAAAAA955555555AAAAAAAAAAAAAAAAAAA95555555),
        .INIT_32 (256'h55556AAAAAAAAAAAAAAAAAAAAA5AA55555556AAAAAAAAAAAAAAAAAAA55555555),
        .INIT_33 (256'h5555AAAAAAAAAAAAAAAAAAAAA956A9555555AAAAAAAAAAAAAAAAAAA955555555),
        .INIT_34 (256'h5556AAAAAAAAAAAAAAAAAAAAA555AA555556AAAAAAAAAAAAAAAAAAA555555555),
        .INIT_35 (256'h555AAAAAAAAAAAAAAAAAAAAA95556A95555AAAAAAAAAAAAAAAAAAA9555555555),
        .INIT_36 (256'h556AAAAAAAAAAAAAAAAAAAAA55555AA5556AAAAAAAAAAAAAAAAAAA5555555555),
        .INIT_37 (256'h55AAAAAAAAAAAAAAAAAAAAA9555556A955AAAAAAAAAAAAAAAAAAA95555555555),
        .INIT_38 (256'h56AAAAAAAAAAAAAAAAAAAAA5555555AA56AAAAAAAAAAAAAAAAAAA55555555555),
        .INIT_39 (256'h5AAAAAAAAAAAAAAAAAAAAA955555556A9AAAAAAAAAAAAAAAAAAA955555555555),
        .INIT_3A (256'h6AAAAAAAAAAAAAAAAAAAAA555555555AAAAAAAAAAAAAAAAAAAAA555555555555),
        .INIT_3B (256'hAAAAAAAAAAAAAAAAAAAAA95555555556AAAAAAAAAAAAAAAAAAA9555555555555),
        .INIT_3C (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3D (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3E (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3F (256'h5555555555555555555555555555555555555555555555555555555555555555))
      HWCB (
        .DOA(hwc_ah_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bh_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

    end
    else if (IMAGE == "SPARTAN3A")
    begin : spartan3_logo
 
      RAMB16_S2_S2 #(
        .INIT_00 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_01 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_02 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_03 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_04 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_05 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_06 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_07 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_08 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_09 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0A (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0B (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0C (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0D (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0E (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_0F (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_10 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_11 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_12 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_13 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_14 (256'hAAAAAAAAA802AA000AAAA80AA802AA00E80002A80002AA00AA0000800AAAAAAA),
        .INIT_15 (256'hAAAAAAAAA003A800C3AAA80EA00FAA00E80000E0000EAA00E800038000EAAAAA),
        .INIT_16 (256'hAAAAAAAAA00FA803ABAAA83A800EA800F8F03FC03C0EA800E00F03AA80FAAAAA),
        .INIT_17 (256'hAAAAAAAAA003A80FAAAAA83A800EA8003AA03A80380EA800F00E03AA803AAAAA),
        .INIT_18 (256'hAAAAAAAA8033AA03AAAAA83A000EA80C3AA03AA0380EA80C300E03AA003AAAAA),
        .INIT_19 (256'hAAAAAAAA80E0EA800EAAA838000EA00E3AA03AA0380EA00C383E03A000FAAAAA),
        .INIT_1A (256'hAAAAAAAA00E0E800FE003830030EA03E0EA03AA8000EA03E0E0E038003EAAAAA),
        .INIT_1B (256'hAAAAAAAA0000EC0FAA8008300F0EA0000EA03AA8000EA0000EA303000FAAAAAA),
        .INIT_1C (256'hAAAAAAAA0000300EAAAAA8003E0E800003A03AA00C0E80000EAA0300FAAAAAAA),
        .INIT_1D (256'hAAAAAAA803A8300EAAAAA8003A0E803A83A03AA00C0E803A83AA0303AAAAAAAA),
        .INIT_1E (256'hAAAAAAA803AA380AAAAAA800EA0E00EA83A03A80380E00EA83AA0203A83AAAAA),
        .INIT_1F (256'hAAAAAAA00EAA0E0003AAA803AA0E00EA83E03A00F80E00EA83A80380003AAAAA),
        .INIT_20 (256'hAAAAAAA803AA8EA00FAAA80EAA0380EAAC203E03E80F80EAA0E803A803FAAAAA),
        .INIT_21 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_22 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_23 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_24 (256'hAAAAAAAA84FFFFFFFFFFFFFFFFFFFFFC30C03FFFFFFFFFFFFFFFFFFC52AAAAAA),
        .INIT_25 (256'hAAAAAAAA053FFFFFFFFFFFFFFFFFF00000C300CFFFFFFFFFFFFFFFF142AAAAAA),
        .INIT_26 (256'hAAAAAAAA813FFFFFFFFFFFFFFFFC000C30C30103FFFFFFFFFFFFFF010AAAAAAA),
        .INIT_27 (256'hAAAAAAAAA04FFFFFFFFFFFFFFFC3400C30C30C0C3FFFFFFFFFFFFF052AAAAAAA),
        .INIT_28 (256'hAAAAAAAAA853FFFFFFFFFFFFFF30000030C0000433FFFFFFFFFFFC1C2AAAAAAA),
        .INIT_29 (256'hAAAAAAAAA813FFFFFFFFFFFFFF10C0C300CC343003FFFFFFFFFFFC10AAAAAAAA),
        .INIT_2A (256'hAAAAAAAAAA04FFFFFFFFFFFFFF00300300CC30C003FFFFFFFFFFC072AAAAAAAA),
        .INIT_2B (256'hAAAAAAAAAA853FFFFFFFFFFFFFF00C331000C34040FFFFFFFFFFC14AAAAAAAAA),
        .INIT_2C (256'hAAAAAAAAAA810FFFFFFFFFFFFFFF031000000F0D003FFFFFFFFF310AAAAAAAAA),
        .INIT_2D (256'hAAAAAAAAAAA04FFFFFFFFFFFFFFFC0C005540030000FFFFFFFFF042AAAAAAAAA),
        .INIT_2E (256'hAAAAAAAAAAA853FFFFFFFFFFFFFC0C0040000005550FFFFFFFFC14AAAAAAAAAA),
        .INIT_2F (256'hAAAAAAAAAAA814FFFFFFFFCCC003000000D5555554CFFFCFFFF010AAAAAAAAAA),
        .INIT_30 (256'hAAAAAAAAAAAA04FFF3CC00300000355555555DDF5403F00FFFCC42AAAAAAAAAA),
        .INIT_31 (256'hAAAAAAAAAAAA81300C00000D5555577FFF3003FC50030013FFC14AAAAAAAAAAA),
        .INIT_32 (256'hAAAAAAAAAAAA81400355555FFC555555555557F14000151000010AAAAAAAAAAA),
        .INIT_33 (256'hAAAAAAAAA8005D0C000000000000000000015FF15554555C00042AAAAAAAAAAA),
        .INIT_34 (256'hAAAAAAAAAAAAA811DDFFFFF777FFFF0000017FFFFDF715515554002AAAAAAAAA),
        .INIT_35 (256'hAAAAAAAAAAAAA814FFFFFFFFFFFFFD000005CCCC4CC3150300C1D555400AAAAA),
        .INIT_36 (256'hAAAAAAAAAAAAAA053FFFFFFFFFFFF00000031111000050100000000000000AAA),
        .INIT_37 (256'hAAAAAAAAAAAAAA813FFFFFFFFFFFF004400000100401500D7042AAAAAAAAAAAA),
        .INIT_38 (256'hAAAAAAAAAAAAAAA34FFFFFFFFFFFF005500000100C014007C10AAAAAAAAAAAAA),
        .INIT_39 (256'hAAAAAAAAAAAAAAA053FFFFFFFFFFC300D400000007F0553F052AAAAAAAAAAAAA),
        .INIT_3A (256'hAAAAAAAAAAAAAAA813FFFFFFFFFF030055400000DFFC154F042AAAAAAAAAAAAA),
        .INIT_3B (256'hAAAAAAAAAAAAAAAA04FFFFFFFFFFFD0155400001FFFC354010AAAAAAAAAAAAAA),
        .INIT_3C (256'hAAAAAAAAAAAAAAAA053FFFFFFFFFFC0155000003FFFF055052AAAAAAAAAAAAAA),
        .INIT_3D (256'hAAAAAAAAAAAAAAAA810FFFFFFFFFFF0155000003FFFFC15742AAAAAAAAAAAAAA),
        .INIT_3E (256'hAAAAAAAAAAAAAAAAA04FFFFFFFFFFC0057000003FFFFC1540AAAAAAAAAAAAAAA),
        .INIT_3F (256'hAAAAAAAAAAAAAAAAA853FFFFFFFFFF0154000003FFFFC0552AAAAAAAAAAAAAAA))
      HWCA (
        .DOA(hwc_al_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bl_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

      RAMB16_S2_S2 #(
        .INIT_00 (256'hAAAAAAAAAAAAAAAAA814FFFFFFFFFF0154000000FFFFF0554AAAAAAAAAAAAAAA),
        .INIT_01 (256'hAAAAAAAAAAAAAAAAAA04FFFFFFFFFF0100000001FFFFF01552AAAAAAAAAAAAAA),
        .INIT_02 (256'hAAAAAAAAAAAAAAAAAA053FFFFFC003FC0000000C3FF3301554AAAAAAAAAAAAAA),
        .INIT_03 (256'hAAAAAAAAAAAAAAAAAA814FFFF0300030005540000000001554AAAAAAAAAAAAAA),
        .INIT_04 (256'hAAAAAAAAAAAAAAAAAAA043330C000000015C055550155415552AAAAAAAAAAAAA),
        .INIT_05 (256'hAAAAAAAAAAAAAAAAAAA00000035000000144155555555555554AAAAAAAAAAAAA),
        .INIT_06 (256'hAAAAAAAAAAAAAA00080555555554000001105555555555555552AAAAAAAAAAAA),
        .INIT_07 (256'hAAAAAA0D5AA00555555555555001000005005555555555555552AAAAAAAAAAAA),
        .INIT_08 (256'hAAAA015575155555555555555000000004005555555555555554AAAAAAAAAAAA),
        .INIT_09 (256'hAA05555555555555555555555400000000005555555555555550AAAAAAAAAAAA),
        .INIT_0A (256'hAAA310055445555541555555540000000000555540C111555544AAAAAAAAAAAA),
        .INIT_0B (256'hAAA015555000000110555555C300000000005555401004115440AAAAAAAAAAAA),
        .INIT_0C (256'hAAA800A222AAA800000000000400000000301555500000040100AAAAAAAAAAAA),
        .INIT_0D (256'hAAAAAAAAAAAAAAAAA82000000000000000000555400200000002AAAAAAAAAAAA),
        .INIT_0E (256'hAAAAAAAAAAAAAAAAAAAAAA8100000000000155550042AAA0000AAAAAAAAAAAAA),
        .INIT_0F (256'hAAAAAAAAAAAAAAAAAAAAAAA04000000000055555110AAAAAAAAAAAAAAAAAAAAA),
        .INIT_10 (256'hAAAAAAAAAAAAAAAAAAAAAAA05000000000155551052AAAAAAAAAAAAAAAAAAAAA),
        .INIT_11 (256'hAAAAAAAAAAAAAAAAAAAAAAA81000000000555554142AAAAAAAAAAAAAAAAAAAAA),
        .INIT_12 (256'hAAAAAAAAAAAAAAAAAAAAAAAA040000000055554010AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_13 (256'hAAAAAAAAAAAAAAAAAAAAAAAA050000000015555052AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_14 (256'hAAAAAAAAAAAAAAAAAAAAAAAA810000000035510142AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_15 (256'hAAAAAAAAAAAAAAAAAAAAAAAAA0400000000004050AAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_16 (256'hAAAAAAAAAAAAAAAAAAAAAAAAA8100000000000052AAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_17 (256'hAAAAAAAAAAAAAAAAAAAAAA87FFFFFFFFFFFFFFFFFC42AAAAAAAAAAAAAAAAAAAA),
        .INIT_18 (256'hAAAAAAAAAAAAAAAAAAAAAA81FFFF3FF00FC0C03FFC42AAAAAAAAAAAAAAAAAAAA),
        .INIT_19 (256'hAAAAAAAAAAAAAAAAAAAAAAA0FFFCCFFFF33CFF3FF10AAAAAAAAAAAAAAAAAAAAA),
        .INIT_1A (256'hAAAAAAAAAAAAAAAAAAAAAAA87FF003C3F3C0C03FF12AAAAAAAAAAAAAAAAAAAAA),
        .INIT_1B (256'hAAAAAAAAAAAAAAAAAAAAAAA81FF3F3CFF3FCFF3FC42AAAAAAAAAAAAAAAAAAAAA),
        .INIT_1C (256'hAAAAAAAAAAAAAAAAAAAAAAAA0FC3F0F00FFCFF3F10AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_1D (256'hAAAAAAAAAAAAAAAAAAAAAAAA83FFFFFFFFFFFFFC52AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_1E (256'hAAAAAAAAAAAAAAAAAAAAAAAA810000000000000142AAAAAAAAAAAAAAAAAAAAAA),
        .INIT_1F (256'hAAAAAAAAAAAAAAAAAAAAAAAAA0555555555555550AAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_20 (256'hAAAAAAAAAAAAAAAAAAAAAAAAA8000140000140002AAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_21 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAA05000050AAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_22 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAA81400042AAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_23 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA0400142AAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_24 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA050050AAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_25 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA814142AAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_26 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA0414AAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_27 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA0550AAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_28 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA8142AAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_29 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA00AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2A (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA02AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2B (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA82AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2C (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2D (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2E (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_2F (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_30 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_31 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_32 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_33 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_34 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_35 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_36 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_37 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_38 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_39 (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3A (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3B (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3C (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3D (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3E (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA),
        .INIT_3F (256'hAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA))
      HWCB (
        .DOA(hwc_ah_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bh_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

    end
    else if (IMAGE == "VIRTEX5")
    begin : spartan3_logo
 
      RAMB16_S2_S2 #(
        .INIT_00 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_01 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_02 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_03 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_04 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_05 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_06 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_07 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_08 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_09 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0A (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0B (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0C (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0D (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0E (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_0F (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_10 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_11 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_12 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_13 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_14 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_15 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_16 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_17 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_18 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_19 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1A (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1B (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1C (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1D (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1E (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_1F (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_20 (256'h555BD55557F557FFFFFFFD7FFFFFFFF557FFFFFFFD57F5FF555555FD55555555),
        .INIT_21 (256'h555735555FD555FFFFFF3D5FFFF3FFF55C3FFFFFCD57B57F555557E555555555),
        .INIT_22 (256'h5555ED557F5555555555CD5555CD5555F1555555FD57F55CD55557F555555755),
        .INIT_23 (256'h55557355FD5555555555BD5555415555F9555555CD57355F955557D555555D55),
        .INIT_24 (256'h55555ED7F55555555555CD5555ED5555CD555555ED57F557F5555C5555557555),
        .INIT_25 (256'h555557FFD555555555553D5555F15555F3555555CD57B557C5557B555555D555),
        .INIT_26 (256'h555555E35555573CFFFFCD5555F155557FD55555FD57F555FD557D555557D555),
        .INIT_27 (256'h555557CED55555FFFFFFBD5555F1555557FCCCF3CD5735557355ED55555F5555),
        .INIT_28 (256'h55555FD7F55555555555CD5555E1555555F3FFFFCD57F5557F55F555557D5555),
        .INIT_29 (256'h55557E55ED55555555553D5555F1555555C55555ED57B5555ED7155555C55555),
        .INIT_2A (256'h5555CD557F5555555555CD5555F155555F155555CD57F5555FDFD5555F155555),
        .INIT_2B (256'h5557B5555ED555555555BD5555C155557C555555CD573555573E555574D55555),
        .INIT_2C (256'h555C1555573557FFFFFFCD5555FD5555F9555555ED57B55557FF5555D3555555),
        .INIT_2D (256'h557F555555FD55FFFFFFFD55557D5557F5555555FD57F55555FD55574D555555),
        .INIT_2E (256'h5555555555555555555555555555555555555555555555555555555F05555555),
        .INIT_2F (256'h555555555555555555555555555555555555555555555555555555F4D5555555),
        .INIT_30 (256'h55555555555555555555555555555555555555555555555555555770D5555555),
        .INIT_31 (256'h55555555555555555555555555555555555555555555555555555DC355555555),
        .INIT_32 (256'h557FFFFFFFFFFFFFFFFFD5555555555555555555555555555555DF0D55555555),
        .INIT_33 (256'h555EAAAAAAAAABAEEEEEB57D5555555555555555555555555557703555555555),
        .INIT_34 (256'h55557777FFFFFBBAAAAAA55F555555555555555555555555555DCC1555555555),
        .INIT_35 (256'h5555555555555555556AA957B5555555555555555555555555DF305555555555),
        .INIT_36 (256'h5555555555555555557AAA55ED55555555555555555555555773015555555555),
        .INIT_37 (256'h5555555555555555555AAAD562D55555555555555555555575CCCD5555555555),
        .INIT_38 (256'h5555555555555555555EAAB552AD55555555555555555555DC30355555555555),
        .INIT_39 (256'h55555555555555555557AAAD562AD555555555555555555D73CC155555555555),
        .INIT_3A (256'h55555555555555555555AAA957A2AD5555555555555555573C30555555555555),
        .INIT_3B (256'h55555555555555555555EAAA55CAAAD5555555555555577FCF03555555555555),
        .INIT_3C (256'h555555555555555555557AAA95688AAB55555555555575FFF0CD555555555555),
        .INIT_3D (256'h555555555555555555557AAAB55EAAAAAF5555555D5DDFFCCF35555555555555),
        .INIT_3E (256'h555555555555555555555AAAAD54AAAAAAAFFDFFD5D7FFFF30D5555555555555),
        .INIT_3F (256'h5555555555555555555556AAAB578AAAAAAABBBD7DFCCFFFC055555555555555))
      HWCA (
        .DOA(hwc_al_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bl_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

      RAMB16_S2_S2 #(
        .INIT_00 (256'h5555555555555555775557AAAA55E2AABAA88B57D7C33FFCCF55555555555555),
        .INIT_01 (256'h5555555555557FFEFFEBBBEA8A957AAAAA8BD57DF00CEFFFCD55555555555555),
        .INIT_02 (256'h55555555555FDFAAAAAAAFAAA8A558AAABD57DFCCFF3FFFC3555555555555555),
        .INIT_03 (256'h5555555555F7AAAAAAAAAAAAAA89562AD57F5FCC30333FFFD555555555555555),
        .INIT_04 (256'h5555555557FAAA888AAAAAAAA8AB57BD5FD7C030C30CCFF05555555555555555),
        .INIT_05 (256'h555555555FEA222AAAAAAAAAAA8AD555F7F00CC333333FFD5555555555555555),
        .INIT_06 (256'h555555556EA8AAA2AAAAAAAAAAA8B57FF30F33330CCCFFFD5555555555555555),
        .INIT_07 (256'h55555555FA8A88AA22ABAEBAE28AA55C30C0CC20CCCF3F355555555555555555),
        .INIT_08 (256'h55555555E8A8AA22ABBAAAAAACB28957033303333030CCD55555555555555555),
        .INIT_09 (256'h55555555EAA2A2AB2FF55555DFFF2B57CC0CCCCCC3F3F3555555555555555555),
        .INIT_0A (256'h55555555A22A2A22D555555555555ED5C3C330303C0CCD555555555555555555),
        .INIT_0B (256'h55555555EA8A8AB9555555555555555570300CCCC3330D555555555555555555),
        .INIT_0C (256'h55555555C8A8A22D55555555555555555CC333030CCCF5555555555555555555),
        .INIT_0D (256'h55555555EAA28AA55555555555555555570CC0CC3330D5555555555555555555),
        .INIT_0E (256'h55555555622A2885555555555555555555030C3330C355555555555555555555),
        .INIT_0F (256'h555555557A8AAAA555555555555555555540CCC0CC8D55555555555555555555),
        .INIT_10 (256'h5555555558A2222D5555555555555555557C303B030D55555555555555555555),
        .INIT_11 (256'h555555555E8AAAA95555555555555555555CC3C03C3555555555555555555555),
        .INIT_12 (256'h5555555557A8888B555555555555555555570C3300D555555555555555555555),
        .INIT_13 (256'h5555555555CAAAAAD5555555555555555555030CC35555555555555555555555),
        .INIT_14 (256'h5555555555732222B55555555555555555557030C15555555555555555555555),
        .INIT_15 (256'h55555555555EAAAAAD55555555555555555570C3055555555555555555555555),
        .INIT_16 (256'h5555555555578C88AA5555555555555555555F0C155555555555555555555555),
        .INIT_17 (256'h5555555555556AAA2AB555555555555555555700D55555555555555555555555),
        .INIT_18 (256'h5555555555555F328AA9555555555555555555F3555555555555555555555555),
        .INIT_19 (256'h55555555555555EBAAAA95555555555555555541555555555555555555555555),
        .INIT_1A (256'h555555555555555E3AAAAB555555555555555555555555555555555555555555),
        .INIT_1B (256'h555555555555555578EAAEB55555555555555555555555555555555555555555),
        .INIT_1C (256'h555555555555555557EBAAAAD555555555555555555555555555555555555555),
        .INIT_1D (256'h55555555555555555557EAAAABF5555555555555555555555555555555555555),
        .INIT_1E (256'h5555555555555555555557EBAEBAFD5555555555555555555555555555555555),
        .INIT_1F (256'h555555555555555555555557EAABAEBF55555555555555555555555555555555),
        .INIT_20 (256'h555555555555555555555555557EF8AAAAFF5555555555555555555555555555),
        .INIT_21 (256'h55555555555555555555555555555555DDD55555555555555555555555555555),
        .INIT_22 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_23 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_24 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_25 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_26 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_27 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_28 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_29 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2A (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2B (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2C (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2D (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2E (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_2F (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_30 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_31 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_32 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_33 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_34 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_35 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_36 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_37 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_38 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_39 (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3A (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3B (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3C (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3D (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3E (256'h5555555555555555555555555555555555555555555555555555555555555555),
        .INIT_3F (256'h5555555555555555555555555555555555555555555555555555555555555555))
      HWCB (
        .DOA(hwc_ah_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bh_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

    end
    else
    begin : nothing
 
      RAMB16_S2_S2
      HWCA (
        .DOA(hwc_al_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bl_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

      RAMB16_S2_S2
      HWCB (
        .DOA(hwc_ah_out),
        .ADDRA(idx0_addr[12:0]),
        .CLKA(clk), 
        .DIA(2'b00),
        .ENA(1'b1), 
        .WEA(1'b0),
        .SSRA(1'b0),
        .DOB(hwc_bh_out),
        .ADDRB(idx1_addr[12:0]), 
        .CLKB(clk), 
        .DIB(2'b00), 
        .ENB(1'b1), 
        .WEB(1'b0),
        .SSRB(1'b0)
      );

    end
  endgenerate

  //******************************************************************//
  // Pipelined output mux for read data...                            //
  //******************************************************************//

  reg           select0;
  reg           select1;
  reg           idx0_maskx_dly;
  reg           idx0_masky_dly;
  reg           idx1_maskx_dly;
  reg           idx1_masky_dly;

  always @(posedge clk)
  begin
    select0 <= idx0_addr[13];
    select1 <= idx1_addr[13];
    data0 <= select0 ? hwc_ah_out : hwc_al_out;
    data1 <= select1 ? hwc_bh_out : hwc_bl_out;
    idx0_maskx_dly <= idx0_maskx;
    idx0_masky_dly <= idx0_masky;
    idx1_maskx_dly <= idx1_maskx;
    idx1_masky_dly <= idx1_masky;
    mask0 <= idx0_maskx_dly || idx0_masky_dly;
    mask1 <= idx1_maskx_dly || idx1_masky_dly;
  end

  //******************************************************************//
  //                                                                  //
  //******************************************************************//

endmodule