///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_addr_gen_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/19 12:03:15 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: This module generates address and burst done signals to the 
//		  controller.
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module vlog_xst_bl4_addr_gen_0
  (
   input                                          clk,
   input                                          rst,
   input                                          addr_rst,
   input                                          addr_inc,
   input                                          r_w,
   output [((`row_address +
             `col_ap_width + `bank_address)-1):0] addr_out,
   output                                         burst_done,
   output reg                                     cnt_roll
   );

   reg [7:0]                                      column_counter;
   reg [1:0]                                      cnt;
   reg                                            burst_done_reg;
   reg                                            burst_done_1_reg;
   reg                                            burst_done_2_reg;
   reg	[(`row_address-1):0]			  row_address1;
   reg                                            cnt_roll_p;
   reg                                            cnt_roll_p2;
   reg                                            rst_r;

   reg [`bank_address-1:0]                        ba_count;

   always @( negedge clk )
     rst_r <= rst;

   assign               addr_out       = {row_address1,
		                                      {`col_ap_width -8{1'b0}},
				                                  column_counter, ba_count};
   
   assign                                         burst_done     =
                                                  burst_done_1_reg;

  always @ (negedge clk) begin
     if(rst_r == 1'b1 || row_address1[5] == 1'b1)
       row_address1 <=  {{(`row_address-3){1'b0}}, 2'b10};
       else if( r_w == 1'b1 && burst_done_1_reg == 1'b0 
			&& burst_done_2_reg == 1'b1)
       row_address1 <=  row_address1 + 8'b10;
     else
       row_address1 <=  row_address1;
   end

   always @ (negedge clk) begin
     if(rst_r == 1'b1)
       ba_count <=  2'b00;
       else if( r_w == 1'b1 && burst_done_1_reg == 1'b0 
			&& burst_done_2_reg == 1'b1)
       ba_count <=  ba_count + 1'b1;
     else
       ba_count <=  ba_count;
   end

   always @ (negedge clk) begin
      if (rst_r == 1'b1 || addr_rst == 1'b1) begin
         column_counter <= 8'b0;
         cnt <= 2'b0;
      end
      else if (addr_inc == 1'b1) begin
         if (cnt == 2'b01)
           cnt <= 2'b0;
         else
           cnt <= cnt + 1'b1;
         if ((cnt == 2'b01)) begin
  	    if (column_counter == 8'b00010000)
              column_counter <= 8'b0;
            else
              column_counter <= column_counter + 4'h4;
         end
         else
           column_counter <= column_counter;
      end
   end


   always @ (negedge clk) begin
      burst_done_reg <= (!rst_r && column_counter[4]);
   end

   always @ (negedge clk) begin
      burst_done_1_reg <= (!rst_r && burst_done_reg );
   end

   always @ (negedge clk) begin
      burst_done_2_reg <= burst_done_1_reg;
   end



   always @ (negedge clk) begin
      cnt_roll_p <= (!rst_r && column_counter[3] &&
                          column_counter[2] && !column_counter[1]
                         && !column_counter[0]); //BL = 4
   end




   always @ (negedge clk) begin
      if (rst_r == 1'b1) begin
         cnt_roll_p2 <= 1'b0;
         cnt_roll    <= 1'b0;
      end
      else begin
         cnt_roll_p2  <= cnt_roll_p;
         cnt_roll     <= cnt_roll_p2;
      end
   end


endmodule
