///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_cmp_data_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/10 07:05:49 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: This module compare the read data with written data and 
//		  generates the error signal in case of data mismatch.
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module  vlog_xst_bl4_cmp_data_0
  (
   input                           clk,
   input                           rst,
   input                           data_valid,
   input [((`data_width*2)-1):0]   lfsr_data,
   input [((`data_width*2)-1):0]   read_data,
   output                          led_error_output,
   output                          data_valid_out
   );

   reg                             led_state;
   reg                             valid;
   reg [((`data_width/8)-1):0]     byte_err;
   reg [((`data_width/8)-1):0]     byte_err1;
   reg                             val_reg;
   reg [((`data_width*2)-1):0]     read_data_reg;
   reg                             rst_r;
   wire                            error;
   wire [15:0]                     lfsr_0;
   wire [15:0]                     lfsr_1;

   wire[15:0]                      data_0;
   wire[15:0]                      data_1;


   always @( posedge clk )
     rst_r <= rst;


   assign                          led_error_output = (led_state == 1'b1) ? 
					                         1'b1 : 1'b0;
   assign                          data_valid_out   =  valid;
   assign     data_0         = read_data_reg[15:0];
   assign     data_1         = read_data_reg[31:16];
   assign     lfsr_0         = lfsr_data[15:0];
   assign     lfsr_1         = lfsr_data[31:16];

   always @ (posedge clk)
     read_data_reg <= read_data;

   always @ (posedge clk) begin
      if (rst_r == 1'b1) begin
         valid   <= 1'b0;
      end
      else begin
         valid   <= data_valid;
      end
   end

   always @ (posedge clk) begin
      if (rst_r == 1'b1)
        val_reg <= 1'b0;
      else
        val_reg <= valid;
   end

   always @ (posedge clk) begin
         byte_err[0]  <= (data_0[7:0] != lfsr_0[7:0]);
         byte_err1[0] <= (data_0[15:8] != lfsr_0[15:8]);
         byte_err[1]  <= (data_1[7:0] != lfsr_1[7:0]);
         byte_err1[1] <= (data_1[15:8] != lfsr_1[15:8]);
   end

   assign error = ((|(byte_err[((`data_width/8)-1):0])) || 
                   (|(byte_err1[((`data_width/8)-1):0]))) && val_reg;

// LED error output
   always @ (posedge clk) begin
      if (rst_r == 1'b1)
        led_state <= 1'b0;  
      else begin
         case(led_state)
           1'b0 : begin
              if (error == 1'b1) begin
                 led_state <= 1'b1; 
		             //synthesis translate_off
                 $display($time, 
		                      " ### LED_ERROR : byte_err= %b , byte_err1= %b ###" 
		                      ,byte_err,byte_err1);
                 $finish;
		             //synthesis translate_on
              end
              else
                led_state <= 1'b0; 
           end
           1'b1 : led_state <= 1'b1;
           default : led_state <= 1'b0;
         endcase
      end
   end

endmodule
