///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_controller_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/03/07 13:51:57 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: This is main controller block. This includes the following
//                features:
//                - The controller state machine that controls the
//                initialization process upon power up, as well as the
//                read, write, and refresh commands.
//                - Accepts and decodes the user commands.
//                - Generates the address and Bank address signals
//                - Generates control signals for other modules, including
//                the control signals for the dqs_en block.
///////////////////////////////////////////////////////////////////////////////
`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module vlog_xst_bl4_controller_0
  (
   input                                        refr_enable,
   input                                        clk,
   input                                        rst0,
   input                                        rst180,
   input [((`row_address + `col_ap_width)-1):0] address,
   input [`bank_address-1:0]                    bank_address,
   input [3:0]                                  command_register,
   input                                        burst_done,
   output                                       ddr_rasb_cntrl,
   output                                       ddr_casb_cntrl,
   output [`bank_address-1:0]                   ddr_ba_cntrl,
   output [`row_address-1:0]                    ddr_address_cntrl,
   output                                       ddr_csb_cntrl,
   output                                       dqs_enable,
   output                                       dqs_reset ,
   output                                       rst_dqs_div_int,
   output                                       cmd_ack,
   output                                       init,
   output					ddr_web_cntrl,
   output                                       ddr_cke_cntrl,
   output reg                                   write_enable,
   output reg                                   rst_calib,
   output reg                                   ddr_ODT_cntrl,
   output reg                                   ar_done,
   input                                        wait_200us,
   output                                       auto_ref_req
   );

   parameter [3:0]                              IDLE = 0,
                                                PRECHARGE = 1,
                                                AUTO_REFRESH =2,
                                                ACTIVE = 3,
                                                FIRST_WRITE =4,
                                                WRITE_WAIT =5,
                                                BURST_WRITE = 6,
                                                PRECHARGE_AFTER_WRITE = 7,
                                                PRECHARGE_AFTER_WRITE_2 = 8,
                                                READ_WAIT =9,
                                                BURST_READ = 10,
                                                ACTIVE_WAIT = 11; 


						
   parameter [1:0]                              INIT_IDLE = 0,
                                                INIT_PRECHARGE = 1,
                                                INIT_LOAD_MODE_REG = 2,
                                                INIT_AUTO_REFRESH =3;

					
   parameter [5:0]                              cntnext = 6'd40;
   reg [3:0]                                    next_state;
   reg [3:0]                                    next_state1;
   reg [3:0]                                    init_next_state;
   reg [3:0]                                    init_next_state1;
   wire                                         ack_reg;
   wire                                         ack_o;
   reg [((`row_address + `col_ap_width)-1):0]   address_reg;
   reg                                          auto_ref;
   reg                                          auto_ref1;
   reg                                          AUTOREF_value;
   reg                                          AUTO_REF_detect1;
   reg [(`max_ref_width-1):0]                   AUTOREF_COUNT;
   reg                                          Auto_Ref_issued;
   wire                                         Auto_Ref_issued_p;
   wire                                         AR_done_p;
   reg [`bank_address-1:0]                      BA_address_reg;
   reg [2:0]                                    burst_length;
   wire [2:0]                                   burst_cnt_max;
   reg [2:0]                                    CAS_COUNT;

   reg [`row_address-1:0]                       column_address_reg;
   reg [`row_address-1:0]                       column_address_reg1;
   wire [`row_address-1:0]                      column_address;
   reg [3:0]                                    command_reg;
   reg [2:0]                                    WR;
   wire                                         ddr_rasb1;
   wire                                         ddr_casb1;
   wire                                         ddr_web1;
   reg                                          ddr_rasb2;
   reg                                          ddr_casb2;
   reg                                          ddr_web2;
   reg [`bank_address-1:0]                      ddr_ba1;
   reg [`row_address-1:0]                       ddr_address1;
   wire [3:0]                                   INIT_COUNT_value;
   reg [3:0]                                    INIT_COUNT;
   reg                                          INIT_DONE;
   wire                                         init_done_value;
   reg                                          init_memory;
   reg                                          init_mem;
   wire                                         initialize_memory;
   reg [6:0]                                    INIT_PRE_COUNT;
   wire [7:0]                                   DLL_RST_COUNT_value;
   reg [7:0]                                    DLL_RST_COUNT;
   reg [(`max_ref_width-1):0]                   REF_FREQ_CNT;
   wire                                         PRECHARGE_CMD;
   wire                                         read_cmd;
   reg                                          read_cmd1;
   reg                                          read_cmd2;
   reg                                          read_cmd3;
   reg [2:0]                                    RCDR_COUNT;
   reg [2:0]                                    RCDW_COUNT;
   wire [2:0]                                   rp_cnt_value;
   reg [5:0]                                    RFC_COUNTER_value;
   reg                                          RFC_COUNT_reg;
   reg                                          AR_Done_reg;
   reg [2:0]                                    RDBURST_END_CNT;
   reg                                          rdburst_end_1;
   reg                                          rdburst_end_2;
   wire                                         rdburst_end;
   reg [2:0]                                    RP_COUNT;
   reg [5:0]                                    RFC_COUNT;
   wire [`row_address-1:0]                      ROW_ADDRESS;
   reg [`row_address-1:0]                       row_address_reg;
   reg                                          rst_dqs_div_r;
   reg                                          rst_dqs_div_r1; //For Reg Dimm
   reg [2:0]                                    wrburst_end_cnt;
   wire                                         wrburst_end;
   reg                                          wrburst_end_1;
   reg                                          wrburst_end_2;
   reg                                          wrburst_end_3;
   reg [2:0]                                    WR_COUNT;
   wire                                         write_cmd_in;
   reg                                          write_cmd1;
   reg                                          write_cmd2;
   reg                                          write_cmd3;
   wire [`row_address - 1:0]                    LMR;
   wire [`row_address - 1:0]                    EMR;
   wire [`row_address - 1:0]                    LMR_DLL_rst;
   wire [`row_address - 1:0]                    LMR_DLL_set;
   reg [2:0]                                    dqs_div_cascount;
   reg [2:0]                                    dqs_div_rdburstcount;
   wire                                         rst_dqs_div_int1;
   reg                                          DQS_enable1;
   reg                                          DQS_enable2;
   reg                                          DQS_enable3;
   reg                                          DQS_reset1_clk0;
   reg                                          DQS_reset2_clk0;
   reg                                          DQS_reset3_clk0;
   reg                                          DQS_enable_int;
   reg                                          DQS_reset_int;
   reg                                          rst180_r;
   reg                                          rst0_r;
   wire                                         ddr_ODT1;
   reg                                          ddr_ODT2;
   wire                                         GO_TO_ACTIVE_value;
   reg                                          GO_TO_ACTIVE;
   reg                                          accept_cmd_in;
   reg  					odt_deassert;
// Following flags are added to resolve the timing erros. Most of the counter
   reg                                          rpCnt0 ;
   reg                                          auto_ref_wait;
   reg                                          auto_ref_wait1;
   reg                                          auto_ref_wait2;
   reg [5:0]                                    count6;

   // Input : COMMAND REGISTER FORMAT
   //          0000  - NOP
   //          0001  - Precharge
   //          0010  - Initialize memory
   //          0100  - Write Request
   //          0110  - Read request

   // Input : Address format
   //   row address  = address((`row_address+ `col_ap_width) -1 : `col_ap_width)
   //   column addrs = address( `row_address-1 : 0)

   assign    ddr_csb_cntrl       = 1'b0;
   assign    ROW_ADDRESS         = address_reg[((`row_address +
                                                 `col_ap_width)-1):
                                               `col_ap_width];
   assign    column_address      = {address_reg[`row_address - 1:11],1'b0,address_reg[9:0]};
   assign    init                = INIT_DONE;
   assign    ddr_rasb_cntrl      = ddr_rasb2;
   assign    ddr_casb_cntrl      = ddr_casb2;
   assign    ddr_web_cntrl       = ddr_web2;
   assign    ddr_address_cntrl   = ddr_address1;
   assign    ddr_ba_cntrl        = ddr_ba1;
   assign    rst_dqs_div_int     = rst_dqs_div_int1;
   assign    EMR                 = `ext_load_mode_register;
   assign    LMR                 = `load_mode_register;
   assign    LMR_DLL_rst         = {LMR[`row_address - 1 : 9],1'b1,LMR[7:0]};
   assign    LMR_DLL_set         = {LMR[`row_address - 1 : 9],1'b0,LMR[7:0]};
   assign    ddr_cke_cntrl       = ~wait_200us;

   always @ (negedge clk)
     begin
        rst180_r <= rst180;
     end

   always @ (posedge clk)
     begin
        rst0_r <= rst0;
     end

//******************************************************************************
// register input commands from the user
//******************************************************************************

  always @ (negedge clk) begin
     command_reg        <= command_register;
     row_address_reg    <= ROW_ADDRESS;
     column_address_reg <= column_address;
     BA_address_reg     <= bank_address;
     address_reg        <= address;
  end

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         burst_length <= 3'b000;
         WR           <= 3'b0;
      end
      else begin
         burst_length  <= LMR[2:0];
         WR            <= `twr_count_value;
      end
   end

   always @( negedge clk ) begin
      if ( rst180_r )
        accept_cmd_in <= 1'b0;
      else if ( next_state == IDLE && ((rpCnt0 && RFC_COUNT_reg &&
                                        !auto_ref_wait && !Auto_Ref_issued)))
        accept_cmd_in <= 1'b1;
      else
        accept_cmd_in <= 1'b0;
   end

   assign PRECHARGE_CMD     = (command_register == 4'b0001 &&
                               accept_cmd_in == 1'b1) ;
   assign initialize_memory = (command_register == 4'b0010);
   assign write_cmd_in      = (command_register == 4'b0100 &&
                               accept_cmd_in == 1'b1) ;
   assign read_cmd          = (command_reg == 4'b0110 &&
                               accept_cmd_in == 1'b1) ;

//Auto Refresh User Command is removed Since Controller is giving 
//Auto refresh commands for every 7.7 us

//**************************************************************************
//write_cmd is used to determine when there is a new write request
//**************************************************************************

// register write_cmd until WRITE command needs to be issued

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         write_cmd1  <= 1'b0;
         write_cmd2  <= 1'b0;
         write_cmd3  <= 1'b0;
      end
      else begin
        if (accept_cmd_in)
          write_cmd1 <= write_cmd_in;
         write_cmd2 <= write_cmd1;
         write_cmd3 <= write_cmd2;
      end
   end

//************************************************************************
// register read cmd until READ command needs to be issued
//************************************************************************

  
 always @ (negedge clk) begin
     if (rst180_r == 1'b1) begin
        read_cmd1      <= 1'b0;
        read_cmd2      <= 1'b0;
        read_cmd3      <= 1'b0;
     end
     else begin
        if (accept_cmd_in)
          read_cmd1       <= read_cmd;
        read_cmd2       <= read_cmd1;
        read_cmd3       <= read_cmd2;
     end
  end
  
//******************************************************************************
// RFC Counter
// an executable command can be issued only after Trfc(70 ns => 70/6.024 = 12 
// cycles for 166MHz,
// 70 ns => 70/10 = 7 cycles for 100MHz )
// after a AUTOREFRESH command is issued
//******************************************************************************

   always @( negedge clk ) begin
      if (rst180_r == 1'b1)
        RFC_COUNT <= 6'b000000;
      else if(next_state == AUTO_REFRESH)
        RFC_COUNT <= RFC_COUNTER_value;
      else if(RFC_COUNT != 6'b000000)
        RFC_COUNT <= RFC_COUNT - 1'b1;
   end

//******************************************************************************
// RP Counter
// an executable command can be issued only after
// Trp(20 ns for a -5 device => 4 cycles)
// after a PRECHARGE command is issued
//******************************************************************************
   assign rp_cnt_value = (next_state == PRECHARGE) ? 3'b100 :
                         (rpCnt0 != 1'b1) ? (RP_COUNT - 1'b1) :
                         3'b000;

   always @( negedge clk ) begin
      if ( rst180_r )
        RCDR_COUNT <= 3'b000;
      else if ( next_state == ACTIVE )
        RCDR_COUNT <= 3'b001; 
      else if ( RCDR_COUNT != 3'b000 )
        RCDR_COUNT <= RCDR_COUNT - 1'b1;
   end

   always @( negedge clk ) begin
      if ( rst180_r )
        RCDW_COUNT <= 3'b000;
      else if ( next_state == ACTIVE)
        RCDW_COUNT <= 3'b001; 
      else if ( RCDW_COUNT != 3'b000 )
        RCDW_COUNT <= RCDW_COUNT - 1'b1;
   end

//******************************************************************************
// WR Counter a PRECHARGE command can be applied only after 2 cycles after a 
// WRITE command has finished executing
//******************************************************************************

   always @(negedge clk) begin
      if (rst180_r)
        WR_COUNT <= 3'b000;
      else
        if (DQS_enable_int)
          WR_COUNT <=  WR ;
        else if (WR_COUNT != 3'b000)
          WR_COUNT <= WR_COUNT - 3'b001;
   end


   
//******************************************************************************
// Auto refresh counter - the design uses AUTO REFRESH the DDR2 SDRAM requires 
// AUTO REFRESH cycles at an average interval of 7.8 us
// Hence, a counter value to obtain a 7.8 us clock for Auto Refresh
// (Refresh Request is raised for every 7.7 us to allow termination for any
// ongoing process) Hence for 200MHz frequency,The Refresh_count_value =
// freq * refresh_time_period = 200*7.7 = 1540 us
//******************************************************************************
   always @ (negedge clk) begin
      if (rst180_r == 1'b1)  begin
         RFC_COUNTER_value <= `rfc_count_value;
         REF_FREQ_CNT      <= `max_ref_cnt;
      end
      else begin
         RFC_COUNTER_value <= `rfc_count_value;
         REF_FREQ_CNT      <= `max_ref_cnt;
         AUTOREF_value   <= (AUTOREF_COUNT == REF_FREQ_CNT);
      end
   end

   always @(negedge clk) begin
      if(rst180_r)
        AUTOREF_COUNT <= `max_ref_width'b0;
      else if(AUTOREF_value)
        AUTOREF_COUNT <= `max_ref_width'b0;
      else
        AUTOREF_COUNT <= AUTOREF_COUNT + 1'b1;
   end
   
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         AUTO_REF_detect1   <= 1'b0;
         auto_ref1               <= 1'b0;
      end
      else begin
         AUTO_REF_detect1   <= AUTOREF_value && INIT_DONE && refr_enable;
         auto_ref1               <= AUTO_REF_detect1;
      end
   end

   assign AR_done_p = (AR_Done_reg == 1'b1) ? 1'b1 : 1'b0;

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         auto_ref_wait <= 1'b0;
         ar_done  <= 1'b0;
         Auto_Ref_issued <= 1'b0;
      end
      else begin
         if (auto_ref1 && !auto_ref_wait)
           auto_ref_wait <= 1'b1;
         else if (Auto_Ref_issued_p)
           auto_ref_wait <= 1'b0;
         else
           auto_ref_wait <= auto_ref_wait;
         ar_done         <= AR_done_p;
         Auto_Ref_issued <= Auto_Ref_issued_p;
      end
   end

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         auto_ref_wait1 <= 1'b0;
         auto_ref_wait2 <= 1'b0;
         auto_ref       <= 1'b0;
      end
      else begin
         if (Auto_Ref_issued_p) begin
            auto_ref_wait1 <= 1'b0;
            auto_ref_wait2 <= 1'b0;
            auto_ref       <= 1'b0;
         end
         else begin
            auto_ref_wait1  <= auto_ref_wait;
            auto_ref_wait2  <= auto_ref_wait1;
            auto_ref        <= auto_ref_wait2;
         end
      end
   end
   assign auto_ref_req = auto_ref_wait;
   assign Auto_Ref_issued_p = (next_state == AUTO_REFRESH);
   
//******************************************************************************
// Common counter for the Initialization sequence
//******************************************************************************
   always @(negedge clk) begin
      if(rst180_r)
        count6 <= 6'b000000;
      else if(init_next_state == INIT_AUTO_REFRESH || init_next_state ==
              INIT_PRECHARGE || init_next_state == INIT_LOAD_MODE_REG)
        count6 <= cntnext;
      else if(count6 != 6'd0)
        count6 <= count6 - 1'b1;
      else
        count6 <= 6'b000000;
   end

//******************************************************************************
// Burst count value counter when there are cosecutive READs or WRITEs
// While doing consecutive READs or WRITEs, the Burst_count value determines
// when the next READ or WRITE command should be issued. The burst length is
// determined while loading the Load Mode Register burst_cnt_max shows the
// number of clock cycles for each burst
//******************************************************************************
   assign burst_cnt_max = (burst_length == 3'b010) ? 3'b010 :
                          (burst_length == 3'b011) ? 3'b100 :
                          3'b000;

//******************************************************************************
// CAS latency counter
// CAS latencies of 3, 4 can be set using Mode register bits M(6:4)
//      M6  M5  M4   CAS latency
//      0    1   1 -    3
//      1    0   0 -    4
//      others     -   reserved
// This design uses a CAS latency of 3 for a clock rate of 200 MHz
//******************************************************************************

   always @( negedge clk) begin
      if(rst180_r)
        CAS_COUNT <= 3'b000;
      else if(next_state == BURST_READ)
        CAS_COUNT <= burst_cnt_max - 1'b1;
      else if(CAS_COUNT != 3'b000)
        CAS_COUNT <= CAS_COUNT - 1'b1;
   end

   always @( negedge clk ) begin
      if(rst180_r)
        RDBURST_END_CNT <= 3'b000;
      else if(CAS_COUNT == 3'b001)
        RDBURST_END_CNT <= burst_cnt_max;
      else if(RDBURST_END_CNT != 3'b000)
        RDBURST_END_CNT <= RDBURST_END_CNT - 1'b1;
   end

   always @( negedge clk ) begin
      if(rst180_r)
        wrburst_end_cnt <= 3'b000;
      else if((next_state == FIRST_WRITE) || (next_state == BURST_WRITE))
        wrburst_end_cnt <= burst_cnt_max;
      else if(wrburst_end_cnt != 3'b000)
        wrburst_end_cnt <= wrburst_end_cnt - 1'b1;
   end


   always @ (negedge clk) begin
      if (rst180_r == 1'b1) 
        rdburst_end_1 <= 1'b0;
      else begin
         rdburst_end_2 <= rdburst_end_1;
         if (burst_done == 1'b1) 
           rdburst_end_1 <= 1'b1;
         else
           rdburst_end_1 <= 1'b0;
      end
   end

   assign rdburst_end   = rdburst_end_1 || rdburst_end_2 ;
   
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) 
        wrburst_end_1 <= 1'b0;
      else begin
         wrburst_end_2  <= wrburst_end_1;
         wrburst_end_3  <= wrburst_end_2;
         if  (burst_done == 1'b1)  
           wrburst_end_1 <= 1'b1;
         else
           wrburst_end_1 <= 1'b0;
      end
   end

   assign wrburst_end = wrburst_end_1 || wrburst_end_2 || wrburst_end_3;

//******************************************************************************
// to generate the Data Strobe enable and reset signal
// The DQS signal needs to be generated center aligned with the data.
// The controller generates the DQS enable signal when the state machine is
// in the FIRST_WRITE state,to take care of the write preamble
//******************************************************************************

   

   assign dqs_enable     = DQS_enable2;
   assign dqs_reset      = DQS_reset2_clk0;

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         DQS_enable_int <= 1'b0;
         DQS_reset_int  <= 1'b0;
      end
      else begin
         DQS_enable_int <= ((next_state == FIRST_WRITE) ||
                            (next_state == BURST_WRITE) ||
                            (wrburst_end_cnt != 3'b000));
         DQS_reset_int  <= (next_state == FIRST_WRITE);
      end
   end

   always @ (posedge clk) begin
      if (rst0_r == 1'b1) begin
         DQS_enable1     <= 1'b0;
         DQS_enable2     <= 1'b0;
         DQS_enable3     <= 1'b0;
         DQS_reset1_clk0 <= 1'b0;
         DQS_reset2_clk0 <= 1'b0;
         DQS_reset3_clk0 <= 1'b0;
      end
      else begin
         DQS_enable1     <= DQS_enable_int;
         DQS_enable2     <= DQS_enable1;
         DQS_enable3     <= DQS_enable2;
         DQS_reset1_clk0 <= DQS_reset_int;
         DQS_reset2_clk0 <= DQS_reset1_clk0;
         DQS_reset3_clk0 <= DQS_reset2_clk0;
      end
   end

//******************************************************************************
//Generating WRITE and READ enable signals
//******************************************************************************


   always @ (negedge clk) begin
      if (rst180_r == 1'b1)
         write_enable <= 1'b0;
      else if(wrburst_end_cnt != 3'b000)
         write_enable <= 1'b1; //write_enable_out;
      else
         write_enable <= 1'b0;
   end

  assign cmd_ack = ack_reg;

   FD ACK_REG_INST1
     (
      .Q(ack_reg),
      .D(ack_o),
      .C(~clk)
      );
   assign ack_o = ((write_cmd_in == 1'b1) || (write_cmd1 == 1'b1) ||
                   (read_cmd == 1'b1) || (read_cmd1 == 1'b1));
   

//******************************************************************************
//  To initialize memory
//******************************************************************************
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         init_memory <= 1'b0;
         INIT_DONE   <= 1'b0;
      end
      else begin
         init_memory <= init_mem;
         INIT_DONE   <= init_done_value && (INIT_COUNT == 4'b1011);
      end
   end

   always @ (negedge clk) begin
      if (initialize_memory) begin
         INIT_PRE_COUNT <= 7'b101_0000;
      end
      else begin
         INIT_PRE_COUNT <= INIT_PRE_COUNT - 7'h1;
      end
   end

   always @( negedge clk ) begin
      if ( rst180_r )
        init_mem <= 1'b0;
      else if ( initialize_memory )
        init_mem <= 1'b1;
      else if ( (INIT_COUNT == 4'b1011) && (count6 == 6'd1) )
        init_mem <= 1'b0;
      else
        init_mem <= init_mem;
   end


// Counter for Memory Initialization sequence

   assign INIT_COUNT_value = (((init_next_state == INIT_PRECHARGE) ||
                               (init_next_state == INIT_LOAD_MODE_REG) ||
                               (init_next_state == INIT_AUTO_REFRESH)) &&
                              init_memory == 1'b1) ? (INIT_COUNT + 1'b1) :
                             INIT_COUNT;

   assign init_done_value =  (DLL_RST_COUNT == 8'b0000_0001) ;


//Counter for DLL Reset complete
   assign DLL_RST_COUNT_value = ((INIT_COUNT == 4'b0011)) ? 8'b1100_1000 :
          (DLL_RST_COUNT != 8'b0000_0001)?
          (DLL_RST_COUNT - 8'b0000_0001): 8'b0000_0001;

//Signal to go directly to ACTIVE state
   assign GO_TO_ACTIVE_value =((write_cmd_in == 1'b1) && (write_cmd1 != 1'b1))
          || ((read_cmd == 1'b1) && (read_cmd1 != 1'b1))
          ? 1'b1 : 1'b0;

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         GO_TO_ACTIVE <= 1'b0;
      end
      else begin
         GO_TO_ACTIVE <= GO_TO_ACTIVE_value;
      end
   end

//******************************************************************************
// Register counter values
//******************************************************************************
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         INIT_COUNT     <= 4'b0000;
         DLL_RST_COUNT  <= 8'b0000_0000;
         RP_COUNT       <= 3'b000;
         RFC_COUNT_reg   <= 1'b0;
         AR_Done_reg     <= 1'b0;
         rpCnt0 <= 1'b1;
      end
      else begin
         INIT_COUNT     <= INIT_COUNT_value;
         DLL_RST_COUNT  <= DLL_RST_COUNT_value;
         RP_COUNT       <= rp_cnt_value;
         if(RFC_COUNT == 6'b000010 ) 
           AR_Done_reg <= 1'b1;
         else
           AR_Done_reg <= 1'b0;
         if(AR_Done_reg == 1'b1)
           RFC_COUNT_reg <= 1'b1;
         else if(INIT_DONE == 1'b1 && init_mem == 1'b0 &&
                 RFC_COUNT == 6'b000000)
           RFC_COUNT_reg <= 1'b1;
         else if (Auto_Ref_issued  == 1'b1)
           RFC_COUNT_reg <= 1'b0;
         else
           RFC_COUNT_reg <= RFC_COUNT_reg;
           rpCnt0  <= (RP_COUNT[2] == 1'b0 && RP_COUNT[1] == 1'b0
                        && !rp_cnt_value[2]);
      end
   end

//******************************************************************************
// To check current state for the address bus
//******************************************************************************
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         next_state    <= IDLE;
      end
      else begin
         next_state    <= next_state1;
      end
   end

//******************************************************************************
// Initialization state machine
//******************************************************************************
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         init_next_state    <= INIT_IDLE;
      end
      else begin
         init_next_state    <= init_next_state1;
      end
   end

   always @ (*) begin
      if (rst180_r == 1'b1)
        init_next_state1 <= INIT_IDLE;
      else begin
         case (init_next_state)
           INIT_IDLE : begin
              if (init_memory == 1'b1) begin
                 case (INIT_COUNT)
                   4'b0000 : begin
                      if(INIT_PRE_COUNT == 7'b000_0001)
                        init_next_state1 <= INIT_PRECHARGE;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0001 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0010 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0011 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0100 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;                      
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0101 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_PRECHARGE;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0110 : begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_AUTO_REFRESH;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b0111: begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_AUTO_REFRESH;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b1000: begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= INIT_IDLE;
                   end
                   4'b1001: begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= init_next_state;
                   end
                   4'b1010: begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_LOAD_MODE_REG;
                      else
                        init_next_state1 <= init_next_state;
                   end
                   4'b1011: begin
                      if (count6 == 6'd1)
                        init_next_state1 <= INIT_IDLE;
                      else
                        init_next_state1 <= init_next_state;
                   end
                   default :
                     init_next_state1 <= INIT_IDLE;
                 endcase
              end
              else
                init_next_state1 <= IDLE;
           end
           INIT_PRECHARGE :
             init_next_state1 <= INIT_IDLE;
           
           INIT_LOAD_MODE_REG :
             init_next_state1 <= INIT_IDLE;
           
           INIT_AUTO_REFRESH :
             init_next_state1 <= INIT_IDLE;
           
           default :
             init_next_state1 <= INIT_IDLE;
         endcase
      end
   end

//******************************************************************************
// Main state machine
//******************************************************************************

   always @ (*) begin
      if (rst180_r == 1'b1)
        next_state1 <= IDLE;
      else begin
         case (next_state)
           IDLE : begin
              if(~init_mem) begin
                 if ( auto_ref == 1'b1  && RFC_COUNT_reg == 1'b1 
                      && rpCnt0 == 1'b1 )
                   next_state1 <= AUTO_REFRESH;
                 else if (PRECHARGE_CMD == 1'b1)
                   next_state1 <= PRECHARGE;
                 else if (GO_TO_ACTIVE == 1'b1)
                   next_state1 <= ACTIVE;
                 else
                   next_state1 <= IDLE;
              end
              else
                next_state1 <= IDLE;
           end
           
           PRECHARGE :
             next_state1 <= IDLE;
           
           AUTO_REFRESH :
             next_state1 <= IDLE;
           
           ACTIVE :
             next_state1 <= ACTIVE_WAIT;
           
           ACTIVE_WAIT : begin
              if(RCDW_COUNT == 3'b000  && write_cmd1)
                next_state1 <= FIRST_WRITE;
              else if (RCDR_COUNT == 3'b000 && read_cmd3)
                next_state1 <= BURST_READ;
              else
                next_state1 <= ACTIVE_WAIT;
           end
           
           FIRST_WRITE : begin
              next_state1 <= WRITE_WAIT;
           end
           
           WRITE_WAIT : begin
              case(wrburst_end)
                1'b1 :
                  next_state1 <= PRECHARGE_AFTER_WRITE;
                1'b0 : begin
                   if (wrburst_end_cnt == 3'b010)
                     next_state1 <= BURST_WRITE;
                   else
                     next_state1 <= WRITE_WAIT;
                end
                default :
                  next_state1 <= WRITE_WAIT;
              endcase
           end
           BURST_WRITE : begin
              next_state1 <= WRITE_WAIT;
           end
           PRECHARGE_AFTER_WRITE : begin
              next_state1 <= PRECHARGE_AFTER_WRITE_2;
           end
           PRECHARGE_AFTER_WRITE_2 : begin
              if(WR_COUNT == 3'd0)
                next_state1 <= PRECHARGE;
              else
                next_state1 <= PRECHARGE_AFTER_WRITE_2;
           end
           READ_WAIT : begin
              case(rdburst_end)
                1'b1 :
                  next_state1 <= PRECHARGE_AFTER_WRITE;
                1'b0 : begin
                   if (CAS_COUNT == 3'b001)
                     next_state1 <= BURST_READ;
                   else
                     next_state1 <= READ_WAIT;
                end
                default :
                  next_state1 <= READ_WAIT;
              endcase
           end
           BURST_READ : begin
              next_state1 <= READ_WAIT;
           end
           default :
             next_state1 <= IDLE;
         endcase
      end
   end

//******************************************************************************
// Address generation logic
//******************************************************************************
   always @( negedge clk ) begin
      if(rst180_r)
        ddr_address1 <= {`row_address{1'b0}};
      else if(init_mem)
        case ( INIT_COUNT )
          4'b0000, 4'b0101 : ddr_address1 <= {`row_address{1'b0}} |
                                             12'h400;
          4'b0001 : ddr_address1 <= {`row_address{1'b0}}; 
          4'b0010 : ddr_address1 <= {`row_address{1'b0}}; 
          4'b0011 : ddr_address1 <= EMR; 
          4'b0100 : ddr_address1 <= LMR_DLL_rst;
          4'b1000 : ddr_address1 <= LMR_DLL_set; 
          4'b1001 : ddr_address1 <= EMR | 12'h380; 
          4'b1010 : ddr_address1 <= EMR & 12'hC7F;
          default : ddr_address1 <= {`row_address{1'b0}};
        endcase
      else if ( next_state == PRECHARGE | init_next_state == INIT_PRECHARGE )
        ddr_address1 <= {`row_address{1'b0}} | 12'h400;
      else if ( next_state == ACTIVE )
        ddr_address1 <= row_address_reg;
      else if ( next_state == BURST_WRITE || next_state == FIRST_WRITE ||
                next_state == BURST_READ )
        ddr_address1 <= column_address_reg1;
      else
        ddr_address1 <= `row_address'b0;
   end

   always @( negedge clk ) begin
      if ( rst180_r )
        ddr_ba1 <= {{`bank_address-1{1'b0}},1'b0};
      else if ( init_mem )
        case ( INIT_COUNT )
          4'b0001 : ddr_ba1 <= (`bank_address'b10);
          4'b0010 : ddr_ba1 <= (`bank_address'b11);
          4'b0011 , 4'b1001 , 4'b1010 : ddr_ba1 <= (`bank_address'b01);
          default : ddr_ba1 <= {{`bank_address-1{1'b0}},1'b0};
        endcase
      else if ( next_state == ACTIVE || next_state == FIRST_WRITE ||
                next_state == BURST_WRITE || next_state == BURST_READ)
        ddr_ba1 <= BA_address_reg;
      else
        ddr_ba1 <= `bank_address'b0;
   end


  always @( negedge clk ) begin
      if ( rst180_r )
        odt_deassert <= 1'b0;
      else if (wrburst_end_3) 
        odt_deassert <= 1'b1;
      else if(!write_cmd3)
        odt_deassert <= 1'b0;
      else
        odt_deassert <= odt_deassert;
   end
   

   
   assign ddr_ODT1 = ( write_cmd3 == 1'b1  && (EMR[6]|EMR[2]) && !odt_deassert )? 1'b1 : 1'b0;  

   
   
//******************************************************************************
//  Register column address
//******************************************************************************
   always @ (negedge clk) begin
      column_address_reg1 <= column_address_reg;
   end



//******************************************************************************
//Pipeline stages for ddr_address and ddr_ba
//******************************************************************************

   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         ddr_ODT2      <= 1'b0;
         ddr_rasb2       <= 1'b1;
         ddr_casb2       <= 1'b1;
         ddr_web2        <= 1'b1;
      end
      else begin
         ddr_ODT2      <= ddr_ODT1;
         ddr_rasb2       <= ddr_rasb1;
         ddr_casb2       <= ddr_casb1;
         ddr_web2        <= ddr_web1;
      end
   end
   
   always @ (negedge clk) begin
      if (rst180_r == 1'b1) begin
         ddr_ODT_cntrl   <= 1'b0;
      end
      else begin
         ddr_ODT_cntrl   <= ddr_ODT2;
      end
   end

//******************************************************************************
// Control signals to the Memory
//******************************************************************************

  assign ddr_rasb1 = ~((next_state == ACTIVE) || (next_state == PRECHARGE) ||
                       (next_state == AUTO_REFRESH) || 
                       (init_next_state ==  INIT_PRECHARGE) || 
                       (init_next_state == INIT_AUTO_REFRESH)  ||
                       (init_next_state == INIT_LOAD_MODE_REG));
   
   assign ddr_casb1 = ~((next_state == BURST_READ) || (next_state == BURST_WRITE)
                        || (next_state == FIRST_WRITE) ||
                        (next_state == AUTO_REFRESH) ||
                        (init_next_state == INIT_AUTO_REFRESH) ||
                        (init_next_state == INIT_LOAD_MODE_REG));
   
   assign ddr_web1  = ~((next_state == BURST_WRITE) || 
			                  (next_state == FIRST_WRITE) || 
			                  (next_state == PRECHARGE) ||
			                  (init_next_state == INIT_PRECHARGE) || 
                        (init_next_state == INIT_LOAD_MODE_REG));


//******************************************************************************
// Register CONTROL SIGNALS outputs
//******************************************************************************

   always @ (negedge clk) begin
      if (rst180_r == 1'b1)
        dqs_div_cascount <= 3'b0;
     else if ((ddr_rasb2 == 1'b1) && (ddr_casb2 == 1'b0) && (ddr_web2 == 1'b1))
       dqs_div_cascount <= burst_cnt_max ;
     else if (dqs_div_cascount != 3'b000)
       dqs_div_cascount <= dqs_div_cascount - 1'b1;
     else
       dqs_div_cascount <= dqs_div_cascount;
   end
   
   always @ (negedge clk) begin
      if (rst180_r == 1'b1)
        dqs_div_rdburstcount <= 3'b000;
      else begin
         if ((dqs_div_cascount == 3'b001) && (burst_length== 3'b010))
           dqs_div_rdburstcount <= 3'b010;
         else if ((dqs_div_cascount == 3'b011) && (burst_length== 3'b011))
           dqs_div_rdburstcount <= 3'b100;
         else begin
            if (dqs_div_rdburstcount != 3'b000)
              dqs_div_rdburstcount <= dqs_div_rdburstcount - 1'b1;
            else
              dqs_div_rdburstcount <= dqs_div_rdburstcount;
         end
      end
   end

   always @ (negedge clk) begin
      if (rst180_r == 1'b1)
        rst_dqs_div_r <= 1'b0;
      else begin
         if (dqs_div_cascount == 3'b001  && burst_length == 3'b010)
           rst_dqs_div_r <= 1'b1;
         else if (dqs_div_cascount == 3'b011  && burst_length == 3'b011)
           rst_dqs_div_r <= 1'b1;
         else if (dqs_div_rdburstcount == 3'b001 && dqs_div_cascount == 3'b000)
           rst_dqs_div_r <= 1'b0;
         else
           rst_dqs_div_r <= rst_dqs_div_r;
      end
   end
   always @ (negedge clk) begin
      rst_dqs_div_r1 <= rst_dqs_div_r;
   end

   always @( negedge clk ) begin
      if(dqs_div_cascount != 3'b0 || dqs_div_rdburstcount != 3'b0 )
        rst_calib <= 1'b1;
      else
        rst_calib <= 1'b0;
   end

   FD  rst_iob_out
     (
      .Q(rst_dqs_div_int1),
     .D(rst_dqs_div_r),
      .C(clk)
      );

endmodule
