 ///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_data_read_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/10 13:28:59 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: RAM8D modules are instantiated for Read data FIFOs. RAM8D is 
//		  each 8 bits or 4 bits depending on number data bits per strobe.
//		  Each strobe  will have two instances, one for rising edge data
//		  and one for falling edge data. 
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"


module vlog_xst_bl4_data_read_0
  (
   input                                    clk90,
   input                                    reset90_r,
   input [(`data_width-1):0]                ddr_dq_in,
   input                                    read_valid_data_1,
   input                                    fifo_00_wr_en,
   input                                    fifo_01_wr_en,
   input                                    fifo_10_wr_en,
   input                                    fifo_11_wr_en,
   input    [3:0]                           fifo_00_wr_addr,
   input    [3:0]                           fifo_01_wr_addr,
   input    [3:0]                           fifo_10_wr_addr,
   input    [3:0]                           fifo_11_wr_addr,
   input                                    dqs0_delayed_col0,
   input                                    dqs0_delayed_col1,
   input                                    dqs1_delayed_col0,
   input                                    dqs1_delayed_col1,
   output     [((`data_width*2)-1):0]       user_output_data,
   output [3:0]                             fifo0_rd_addr_val,
   output [3:0]                             fifo1_rd_addr_val
   );

   reg                                      read_valid_data_1_r;
   reg                                      read_valid_data_1_r1;
   reg                                      read_valid_data_1_r2;
   reg                                      reset90_reg;
   reg [3:0]                             fifo00_rd_addr_r;
   reg  [3:0]                            fifo01_rd_addr_r;
   reg [3:0]                             fifo10_rd_addr_r;
   reg  [3:0]                            fifo11_rd_addr_r;
   reg [`DatabitsPerStrobe-1:0]           fifo_00_data_out_r;
   reg [`DatabitsPerStrobe-1:0]           fifo_01_data_out_r;
   reg [`DatabitsPerStrobe-1:0]           fifo_10_data_out_r;
   reg [`DatabitsPerStrobe-1:0]           fifo_11_data_out_r;
   reg [((`data_width*2)-1):0]              first_sdr_data;
   wire [3:0]                               fifo00_rd_addr;
   wire [3:0]                               fifo01_rd_addr;
   wire [`DatabitsPerStrobe-1:0]        fifo_00_data_out;
   wire [`DatabitsPerStrobe-1:0]       fifo_01_data_out;
   wire [`DatabitsPerStrobe-1:0]        fifo_10_data_out;
   wire [`DatabitsPerStrobe-1:0]       fifo_11_data_out;
   wire                                 dqs0_delayed_col0_n;
   wire                                dqs0_delayed_col1_n;
   wire                                 dqs1_delayed_col0_n;
   wire                                dqs1_delayed_col1_n;


   assign dqs0_delayed_col0_n  = ~ dqs0_delayed_col0;
   assign dqs0_delayed_col1_n  = ~ dqs0_delayed_col1;
   assign dqs1_delayed_col0_n  = ~ dqs1_delayed_col0;
   assign dqs1_delayed_col1_n  = ~ dqs1_delayed_col1;
   assign user_output_data    = first_sdr_data;
   assign fifo0_rd_addr_val   = fifo01_rd_addr;
   assign fifo1_rd_addr_val   = fifo00_rd_addr;

   always @( posedge clk90 )
      reset90_reg <= reset90_r;

   always@(posedge clk90) begin
      fifo_00_data_out_r <= fifo_00_data_out;
      fifo_01_data_out_r <= fifo_01_data_out;
   end
   always@(posedge clk90) begin
      fifo_10_data_out_r <= fifo_10_data_out;
      fifo_11_data_out_r <= fifo_11_data_out;
   end
   always@(posedge clk90) begin
      fifo00_rd_addr_r <= fifo00_rd_addr;
      fifo01_rd_addr_r <= fifo01_rd_addr;
   end
   always@(posedge clk90) begin
      fifo10_rd_addr_r <= fifo00_rd_addr;
      fifo11_rd_addr_r <= fifo01_rd_addr;
   end

   always @(posedge clk90)begin
      if(reset90_reg)begin
         first_sdr_data   <= {(`data_width*2){1'b0}};
         read_valid_data_1_r <= 1'b0;
         read_valid_data_1_r1 <= 1'b0;
         read_valid_data_1_r2 <= 1'b0;
      end
      else begin
         read_valid_data_1_r <= read_valid_data_1;
         read_valid_data_1_r1 <= read_valid_data_1_r;
         read_valid_data_1_r2 <= read_valid_data_1_r1;
         if(read_valid_data_1_r1)begin
            first_sdr_data  <= {                                 fifo_10_data_out_r,                                 fifo_00_data_out_r,                                fifo_11_data_out_r,                                fifo_01_data_out_r};
         end
      end
   end


   vlog_xst_bl4_rd_gray_cntr fifo0_rd_addr_inst
      (
	     .clk      (clk90),
	     .reset    (reset90_r),
	     .cnt_en   (read_valid_data_1),
       .rgc_gcnt (fifo00_rd_addr)
      );

   vlog_xst_bl4_rd_gray_cntr fifo1_rd_addr_inst
      (
	     .clk	  (clk90),
	     .reset    (reset90_r),
	     .cnt_en   (read_valid_data_1),
	     .rgc_gcnt (fifo01_rd_addr)
       );


   vlog_xst_bl4_RAM8D_0 strobe0
      (
       .DOUT(fifo_00_data_out[`DatabitsPerStrobe-1:0]),
       .WADDR(fifo_00_wr_addr[3:0]),
       .DIN(ddr_dq_in[7:0]), .RADDR(fifo00_rd_addr_r[3:0]),
       .WCLK0(dqs0_delayed_col0), .WCLK1(dqs0_delayed_col1),
       .WE(fifo_00_wr_en)
       );
   vlog_xst_bl4_RAM8D_0 strobe0_n
      (
       .DOUT(fifo_01_data_out[`DatabitsPerStrobe-1:0]),
       .WADDR(fifo_01_wr_addr[3:0]),
       .DIN(ddr_dq_in[7:0]), .RADDR(fifo01_rd_addr_r[3:0]),
       .WCLK0(dqs0_delayed_col0_n), .WCLK1(dqs0_delayed_col1_n),
       .WE(fifo_01_wr_en)
       );
   vlog_xst_bl4_RAM8D_1 strobe1
      (
       .DOUT(fifo_10_data_out[`DatabitsPerStrobe-1:0]),
       .WADDR(fifo_10_wr_addr[3:0]),
       .DIN(ddr_dq_in[15:8]), .RADDR(fifo10_rd_addr_r[3:0]),
       .WCLK0(dqs1_delayed_col0), .WCLK1(dqs1_delayed_col1),
       .WE(fifo_10_wr_en)
       );
   vlog_xst_bl4_RAM8D_1 strobe1_n
      (
       .DOUT(fifo_11_data_out[`DatabitsPerStrobe-1:0]),
       .WADDR(fifo_11_wr_addr[3:0]),
       .DIN(ddr_dq_in[15:8]), .RADDR(fifo11_rd_addr_r[3:0]),
       .WCLK0(dqs1_delayed_col0_n), .WCLK1(dqs1_delayed_col1_n),
       .WE(fifo_11_wr_en)
       );

endmodule
