///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_data_read_controller_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/08 16:03:09 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: This module has instantiations fifo_0_wr_en, fifo_1_wr_en, 
//		  dqs_delay and wr_gray_cntr.
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module vlog_xst_bl4_data_read_controller_0
  (
   input                           clk90,
   input                           reset_r,
   input                           reset90_r,
   input                           rst_dqs_div_in,
   input [4:0]                     delay_sel,
   input [3:0]                     fifo0_rd_addr,
   input [3:0]                     fifo1_rd_addr,
   input                              dqs_int_delay_in0,
   input                              dqs_int_delay_in1,
   output                           fifo_00_wr_en_val,
   output                           fifo_01_wr_en_val,
   output                           fifo_10_wr_en_val,
   output                           fifo_11_wr_en_val,
   output     [3:0]                 fifo_00_wr_addr_val,
   output     [3:0]                 fifo_01_wr_addr_val,
   output     [3:0]                 fifo_10_wr_addr_val,
   output     [3:0]                 fifo_11_wr_addr_val,
   output                           dqs0_delayed_col0_val,
   output                           dqs0_delayed_col1_val,
   output                           dqs1_delayed_col0_val,
   output                           dqs1_delayed_col1_val,
   output                          u_data_val,
   output                          read_valid_data_1_val
  );

   wire [(`data_strobe_width-1):0] dqs_delayed_col1;
   wire [(`data_strobe_width-1):0] dqs_delayed_col0;
   wire                            fifo_00_empty;
   wire                            fifo_01_empty;
   wire                            read_valid_data_0_1;
   wire                            rst_dqs_div;
   reg                             read_valid_data_r;
   reg                             read_valid_data_r1;
   reg [3:0]                       fifo_00_wr_addr_d;
   reg [3:0]                       fifo_00_wr_addr_2d;
   reg [3:0]                       fifo_00_wr_addr_3d;
   reg [3:0]                       fifo_01_wr_addr_d;
   reg [3:0]                       fifo_01_wr_addr_2d;
   reg [3:0]                       fifo_01_wr_addr_3d;
   reg                             u_data_val_r;
   reg                             reset90_reg;
   wire [3:0]                       fifo_00_wr_addr;
   wire [3:0]                       fifo_01_wr_addr;
   wire [3:0]                       fifo_10_wr_addr;
   wire [3:0]                       fifo_11_wr_addr;
   wire                             dqs0_delayed_col0;
   wire                             dqs0_delayed_col1;
   wire                             dqs1_delayed_col0;
   wire                             dqs1_delayed_col1;
   wire                             fifo_00_wr_en;
   wire                             fifo_01_wr_en;
   wire                             fifo_10_wr_en;
   wire                             fifo_11_wr_en;
   wire                             rst_dqs_delay_0_n;
   wire                             rst_dqs_delay_1_n;
   wire                             dqs0_delayed_col0_n;
   wire                             dqs0_delayed_col1_n;
   wire                             dqs1_delayed_col0_n;
   wire                             dqs1_delayed_col1_n;




   always @( posedge clk90 )
     reset90_reg <= reset90_r;

   assign                         dqs0_delayed_col0     = dqs_delayed_col0[0];
   assign                         dqs0_delayed_col1     = dqs_delayed_col1[0];
   assign                         dqs1_delayed_col0     = dqs_delayed_col0[1];
   assign                         dqs1_delayed_col1     = dqs_delayed_col1[1];
   assign                         fifo_00_wr_addr_val   = fifo_00_wr_addr;
   assign                         fifo_01_wr_addr_val   = fifo_01_wr_addr;
   assign                         fifo_10_wr_addr_val   = fifo_10_wr_addr;
   assign                         fifo_11_wr_addr_val   = fifo_11_wr_addr;
   assign                         fifo_00_wr_en_val     =    fifo_00_wr_en;
   assign                         fifo_01_wr_en_val     =    fifo_01_wr_en;
   assign                         fifo_10_wr_en_val     =    fifo_10_wr_en;
   assign                         fifo_11_wr_en_val     =    fifo_11_wr_en;
   assign                         dqs0_delayed_col0_val = dqs0_delayed_col0;
   assign                         dqs0_delayed_col1_val = dqs0_delayed_col1;
   assign                         dqs1_delayed_col0_val = dqs1_delayed_col0;
   assign                         dqs1_delayed_col1_val = dqs1_delayed_col1;
   assign                         dqs0_delayed_col0_n   = ~ dqs0_delayed_col0;
   assign                         dqs0_delayed_col1_n   = ~ dqs0_delayed_col1;
   assign                         dqs1_delayed_col0_n   = ~ dqs1_delayed_col0;
   assign                         dqs1_delayed_col1_n   = ~ dqs1_delayed_col1;

  assign                         fifo_00_empty         = (fifo0_rd_addr
                                                          == fifo_00_wr_addr_3d) ? 
						                                             1'b1 : 1'b0;
  assign                         fifo_01_empty         = (fifo1_rd_addr ==
                                                          fifo_01_wr_addr_3d) ? 
						                     1'b1 : 1'b0;
  assign                         read_valid_data_0_1   = ((~fifo_00_empty)
                                                          & (~fifo_01_empty));
  assign                         read_valid_data_1_val = (read_valid_data_0_1);
  assign                         u_data_val            = u_data_val_r;

   always@(posedge clk90) begin
      fifo_00_wr_addr_d <= fifo_00_wr_addr;
      fifo_01_wr_addr_d <= fifo_01_wr_addr;
      fifo_00_wr_addr_2d <= fifo_00_wr_addr_d;
      fifo_01_wr_addr_2d <= fifo_01_wr_addr_d;
      fifo_00_wr_addr_3d <= fifo_00_wr_addr_2d;
      fifo_01_wr_addr_3d <= fifo_01_wr_addr_2d;
   end

   always@(posedge clk90) begin
      if(reset90_reg) begin
         u_data_val_r <= 1'b0;
         read_valid_data_r <= 1'b0;
         read_valid_data_r1 <= 1'b0;
      end
      else begin
         read_valid_data_r <= read_valid_data_0_1;
         read_valid_data_r1 <= read_valid_data_r;
         u_data_val_r <= read_valid_data_r1;
      end
   end

   vlog_xst_bl4_dqs_delay rst_dqs_div_delayed1
     (
      .clk_in(rst_dqs_div_in),
      .sel_in(delay_sel),
      .clk_out(rst_dqs_div)
      );


//DQS Internal Delay Circuit implemented in LUTs


   vlog_xst_bl4_dqs_delay dqs_delay0_col0
      (
       .clk_in(dqs_int_delay_in0),
       .sel_in(delay_sel),
       .clk_out(dqs_delayed_col0[0])
       );
   vlog_xst_bl4_dqs_delay dqs_delay0_col1
      (
       .clk_in(dqs_int_delay_in0),
       .sel_in(delay_sel),
       .clk_out(dqs_delayed_col1[0])
       );
   vlog_xst_bl4_dqs_delay dqs_delay1_col0
      (
       .clk_in(dqs_int_delay_in1),
       .sel_in(delay_sel),
       .clk_out(dqs_delayed_col0[1])
       );
   vlog_xst_bl4_dqs_delay dqs_delay1_col1
      (
       .clk_in(dqs_int_delay_in1),
       .sel_in(delay_sel),
       .clk_out(dqs_delayed_col1[1])
       );

// FIFO write enables

   vlog_xst_bl4_fifo_0_wr_en_0 fifo_00_wr_en_inst
      (
       .clk(dqs0_delayed_col1_n),
       .reset(reset_r),
       .din(rst_dqs_div),
       .rst_dqs_delay_n(rst_dqs_delay_0_n),
       .dout(fifo_00_wr_en)
       );
   vlog_xst_bl4_fifo_1_wr_en_0 fifo_01_wr_en_inst
      (
       .clk(dqs0_delayed_col0),
       .rst_dqs_delay_n(rst_dqs_delay_0_n),
       .reset(reset_r),
       .din(rst_dqs_div),
       .dout(fifo_01_wr_en)
       );
   vlog_xst_bl4_fifo_0_wr_en_0 fifo_10_wr_en_inst
      (
       .clk(dqs1_delayed_col1_n),
       .reset(reset_r),
       .din(rst_dqs_div),
       .rst_dqs_delay_n(rst_dqs_delay_1_n),
       .dout(fifo_10_wr_en)
       );
   vlog_xst_bl4_fifo_1_wr_en_0 fifo_11_wr_en_inst
      (
       .clk(dqs1_delayed_col0),
       .rst_dqs_delay_n(rst_dqs_delay_1_n),
       .reset(reset_r),
       .din(rst_dqs_div),
       .dout(fifo_11_wr_en)
       );

//write pointer gray counter instances

   vlog_xst_bl4_wr_gray_cntr fifo_00_wr_addr_inst
      (
       .clk(dqs0_delayed_col1),
       .reset(reset_r),
       .cnt_en(fifo_00_wr_en),
       .wgc_gcnt(fifo_00_wr_addr)
       );
   vlog_xst_bl4_wr_gray_cntr fifo_01_wr_addr_inst
      (
       .clk(dqs0_delayed_col0_n),
       .reset(reset_r),
       .cnt_en(fifo_01_wr_en),
       .wgc_gcnt(fifo_01_wr_addr)
       );
   vlog_xst_bl4_wr_gray_cntr fifo_10_wr_addr_inst
      (
       .clk(dqs1_delayed_col1),
       .reset(reset_r),
       .cnt_en(fifo_10_wr_en),
       .wgc_gcnt(fifo_10_wr_addr)
       );
   vlog_xst_bl4_wr_gray_cntr fifo_11_wr_addr_inst
      (
       .clk(dqs1_delayed_col0_n),
       .reset(reset_r),
       .cnt_en(fifo_11_wr_en),
       .wgc_gcnt(fifo_11_wr_addr)
       );

endmodule
