///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: data_write.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/20 10:35:45 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: Data write operation performed through the pipelines in this 
//		  module.
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module  vlog_xst_bl4_data_write_0
  (
   input     [((`data_width*2)-1):0]  user_input_data,
   input [((`data_mask_width*2)-1):0] user_data_mask,
   input                              clk90,
   input                              write_enable,
   output reg                         write_en_val,
   output [((`data_width)-1):0]       write_data_falling,
   output [((`data_width)-1):0]       write_data_rising,
   output [((`data_mask_width)-1):0]  data_mask_f,
   output [((`data_mask_width)-1):0]  data_mask_r
   );

   reg                                write_en_P1;
   reg                                write_en_P2;
   reg                                write_en_P3;
   reg                                write_en_int;
   reg [((`data_width*2)-1):0]        write_data;
   reg [((`data_width*2)-1):0]        write_data1;
   reg [((`data_width*2)-1):0]        write_data2;
   reg [((`data_width*2)-1):0]        write_data3;
   reg [((`data_width*2)-1):0]        write_data4;
   reg [((`data_width*2)-1):0]        write_data5;
   reg [((`data_width*2)-1):0]        write_data6;
   reg [((`data_width*2)-1):0]        write_data7;
   reg [((`data_width*2)-1):0]        write_data_reg_dimm;  
   reg [((`data_width)-1):0]          write_data270;
   reg [((`data_width)-1):0]          write_data270_1;
   reg [((`data_width)-1):0]          write_data270_2;
   reg [((`data_width)-1):0]          write_data270_3;
   reg [((`data_width)-1):0]          write_data270_4; 
   reg [((`data_width)-1):0]          write_data270_5; 
   reg [((`data_mask_width*2)-1):0]   write_data_m1;
   reg [((`data_mask_width*2)-1):0]   write_data_m2;
   reg [((`data_mask_width*2)-1):0]   write_data_m3;
   reg [((`data_mask_width*2)-1):0]   write_data_m4;
   reg [((`data_mask_width*2)-1):0]   write_data_m5;
   reg [((`data_mask_width*2)-1):0]   write_data_m6;
   reg [((`data_mask_width*2)-1):0]   write_data_m7;   
   reg [((`data_mask_width*2)-1):0]   write_data_mask;
   reg [((`data_mask_width*2)-1):0]   write_data_mask_reg_dimm; 
   reg [((`data_mask_width)-1):0]     write_data_m270;
   reg [((`data_mask_width)-1):0]     write_data_m270_1;
   reg [((`data_mask_width)-1):0]     write_data_m270_2;
   reg [((`data_mask_width)-1):0]     write_data_m270_3;
   reg [((`data_mask_width)-1):0]     write_data_m270_4; 
   reg [((`data_mask_width)-1):0]     write_data_m270_5; 

   wire [((`data_width*2)-1):0]       write_data0;
   wire [((`data_mask_width*2)-1):0]  write_data_m0;

   assign                             write_data0 = user_input_data;
   assign                             write_data_m0 = user_data_mask;

   always@(posedge clk90) begin
      write_data1 <= write_data0;
      write_data2 <= write_data1;
      write_data3 <= write_data2;
      write_data4 <= write_data3;
   end

   always@(posedge clk90) begin
      write_data_m1 <= write_data_m0;
      write_data_m2 <= write_data_m1;
      write_data_m3 <= write_data_m2;
      write_data_m4 <= write_data_m3;
   end
   
   always@(posedge clk90) begin
      write_data5   <=  write_data4;
      write_data6   <= write_data5;
      write_data7   <= write_data6; 
   end
   
   always@(posedge clk90) begin
      write_data_m5   <=  write_data_m4;
      write_data_m6   <= write_data_m5;
      write_data_m7   <= write_data_m6;   
   end
   always@(posedge clk90) begin
      write_data  <= write_data6;
      write_data_reg_dimm <=  write_data;          
   end

   always@(posedge clk90) begin
      write_data_mask        <= write_data_m6;
      write_data_mask_reg_dimm <=  write_data_mask;   
   end

   always@(negedge clk90)
     write_data270  <=    write_data4[(`data_width*2)-1 : `data_width] ;


   always@(negedge clk90)
     write_data_m270 <=    write_data_m4[(`data_mask_width*2)-1:`data_mask_width];

   always@ (negedge clk90) begin
      write_data270_1   <= write_data270;
      write_data_m270_1 <= write_data_m270;
      write_data270_2   <= write_data270_1;
      write_data_m270_2 <= write_data_m270_1;
      write_data270_3   <= write_data270_2;
      write_data_m270_3 <= write_data_m270_2;
      write_data270_4   <= write_data270_3;    
      write_data_m270_4 <= write_data_m270_3;  
      write_data270_5   <= write_data270_4;    
      write_data_m270_5 <= write_data_m270_4;  
   end


   assign write_data_rising  = write_data270_2; //write_data270_3;

   assign write_data_falling = write_data[(`data_width-1):0];


   assign data_mask_r = write_data_m270_2; //write_data_m270_3;

   assign data_mask_f = write_data_mask[(`data_mask_width-1):0];


// write enable for data path
   always@(posedge clk90) begin
     write_en_P1 <= write_enable;
     write_en_P2 <= write_en_P1;
     write_en_P3 <= write_en_P2;
   end

   always@(negedge clk90) begin
     write_en_int   <= write_en_P2; 
     write_en_val   <= write_en_P1; 
   end

endmodule
