///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  /   Vendor			: Xilinx
// \   \   \/    Version		: $Name: mig_v1_7_b7 $
//  \   \        Application		: MIG
//  /   /        Filename		: vlog_xst_bl4_ddr2_test_bench_0.v
// /___/   /\    Date Last Modified	: $Date: 2007/02/19 12:03:15 $
// \   \  /  \   Date Created		: Mon May 2 2005
//  \___\/\___\
// Device	: Spartan-3/3A
// Design Name	: DDR2 SDRAM
// Purpose	: This module generate the commands, address and data associated
//                 with a write and a read command.
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/100ps
`include "../source/sdramtst/vlog_xst_bl4_parameters_0.v"

module    vlog_xst_bl4_ddr2_test_bench_0
  (
   input                                          fpga_clk,
   input                                          fpga_rst90,
   input                                          fpga_rst180,
   input                                          clk90,
   input                                          INIT_DONE,
   input                                          ar_done,
   input                                          u_ack,
   input                                          u_data_val,
   input [(2*`data_width)-1:0]                    u_data_o,
   input                                          auto_ref_req,
   output                                         burst_done,
   output [((`row_address +
             `col_ap_width + `bank_address)-1):0] u_addr,
   output [3:0]                                   u_cmd,
   output [(2*`data_width)-1:0]                   u_data_i,
   output [((`data_mask_width*2)-1):0]            u_data_m,
   output                                         led_error_output,
   output                                         data_valid_out
   );

   wire                                           clk;
   wire                                           addr_inc;
   wire                                           addr_rst;
   wire                                           cmd_ack;
   wire                                           cnt_roll;
   wire                                           data_valid;
   wire                                           lfsr_rst_r1;
   reg                                            lfsr_rst_r;
   wire                                           lfsr_rst_w;
   wire                                           r_w;
   wire [(2*`data_width)-1:0]                     lfsr_data_w;
   wire [(2*`data_width)-1:0]                     lfsr_data_r;
   wire [((`data_mask_width*2)-1):0]              lfsr_data_m_r;
   wire [((`data_mask_width*2)-1):0]              lfsr_data_m_w;
   wire [((`row_address + 
           `col_ap_width + `bank_address)-1):0]   addr_out;
   reg                                            rst90_r;
   reg                                            lfsr_ena_r;
   reg                                            lfsr_ena_w;
   reg [1:0]                                      RTT;
   reg                                            u_dat_flag;
   wire                                           u_dat_fl;
   

// Output : COMMAND REGISTER FORMAT
//          0000  - NOP
//          0001  - Precharge
//          0010  - Initialize memory
//          0100  - Write Request
//          0110  - Read request

// Output : Address format
//   row address  = address((`row_address + `col_ap_width + `bank_address) -1 
//					  : (`col_ap_width + `bank_address))
//   column addrs = address((`row_address + `bank_address)-1 : `bank_address)
//   Bank addrs   = address(`bank_address - 1 : 0)

   assign              clk         = fpga_clk;
   assign              cmd_ack     = u_ack;
   assign              data_valid  = u_data_val;
   assign              u_addr[((`row_address +  `col_ap_width + 
                                `bank_address) -1):0] =  addr_out
                       [((`row_address +  `col_ap_width + `bank_address)-1):0];
   assign              u_data_i     = lfsr_data_w;
   assign              u_data_m     = lfsr_data_m_w;
   
   always @ (posedge clk) begin
`ifdef ODT_CMD
      RTT <= 2'b01;
`else
      RTT <= 2'b00;
`endif
   end

   always @ (posedge clk90) begin
      rst90_r <= fpga_rst90;
   end

   always @ (posedge clk90) begin  
      if (rst90_r == 1'b1)
        lfsr_ena_r <= 1'b0;
      else if (u_data_val == 1'b1)
        lfsr_ena_r <= 1'b1;
      else
        lfsr_ena_r <= 1'b0;
   end
   
   always @ (posedge clk90) begin  
      if (rst90_r == 1'b1)
        lfsr_ena_w <= 1'b0;
      else if ((r_w == 1'b0) && (u_ack == 1'b1))
        lfsr_ena_w <= 1'b1;
      else
        lfsr_ena_w <= 1'b0;
   end

   always@(posedge clk90) begin
      if(rst90_r == 1'b1)
        u_dat_flag <= 1'b0;
      else
        u_dat_flag  <= cmd_ack;
   end

   assign u_dat_fl = { cmd_ack && !u_dat_flag && r_w};
   assign lfsr_rst_r1 = u_dat_fl ;

   always@(posedge clk90) begin
      if(rst90_r == 1'b1)
        lfsr_rst_r <= 1'b0;
      else
        lfsr_rst_r  <= lfsr_rst_r1;
   end

   vlog_xst_bl4_addr_gen_0  INST1
      (
       .clk(clk),
       .rst(fpga_rst180),
       .addr_rst(addr_rst),
       .addr_inc(addr_inc),
       .addr_out(addr_out),
       .burst_done(burst_done),
       .r_w(r_w),
       .cnt_roll(cnt_roll)
        );

   vlog_xst_bl4_cmd_fsm_0  INST_2
     (
      .clk(clk),
      .clk90(clk90),
      .auto_ref_req(auto_ref_req),
      .cmd_ack(cmd_ack),
      .cnt_roll(cnt_roll),
      .r_w(r_w),
      .refresh_done(ar_done),
      .rst90(fpga_rst90),
      .rst180(fpga_rst180),
      .init_val(INIT_DONE),
      .addr_inc(addr_inc),
      .addr_rst(addr_rst),
      .u_cmd(u_cmd),
      .lfsr_rst(lfsr_rst_w)
      );

   vlog_xst_bl4_cmp_data_0   INST3
     (
      .clk(clk90),
      .data_valid(data_valid),
      .lfsr_data(lfsr_data_r),
      .read_data(u_data_o),
      .rst(fpga_rst90),
      .led_error_output(led_error_output),
      .data_valid_out  (data_valid_out)
       );

   vlog_xst_bl4_lfsr32_0     INST5
     (
      .clk(clk90),
      .rst(fpga_rst90),
      .lfsr_rst(lfsr_rst_w),
      .lfsr_ena(lfsr_ena_w),
      .lfsr_data_m(lfsr_data_m_w),
      .lfsr_out(lfsr_data_w)
       );

   vlog_xst_bl4_lfsr32_0     INST7
     (
      .clk(clk90),
      .rst(fpga_rst90),
      .lfsr_rst(lfsr_rst_r),
      .lfsr_ena(lfsr_ena_r),
      .lfsr_data_m(lfsr_data_m_r),
      .lfsr_out(lfsr_data_r)
      );
endmodule
